/* $Id$ */

/* Copyright (C) SGS-THOMSON Microelectronics Ltd 1996 */

/* derived from extlib */

#ifndef __compint_h
#define __compint_

/*{{{  internal type definitions */
typedef int                BOOL;

typedef void *             POINTER;
typedef const void *       VALPOINTER;

typedef char               CHAR;
typedef unsigned char      BYTE;

typedef signed int         INT;
typedef signed char        INT8;
typedef signed short int   INT16;
typedef signed long int    INT32;

typedef unsigned int       UINT;
typedef unsigned char      UINT8;
typedef unsigned short int UINT16;
typedef unsigned long int  UINT32;

typedef BOOL BOOLEAN;

typedef UINT   BIT;
typedef UINT8  BIT8;
typedef UINT16 BIT16;
typedef UINT32 BIT32;

typedef UINT   WORD;
typedef UINT8  WORD8;
typedef UINT16 WORD16;
typedef UINT32 WORD32;

typedef union float_convert_u
        {
          BIT32 __i;
          int   __si;
          float __f;
        } float_convert_t;

typedef union double_convert_u
        {
          BIT32  __i[2];
          int    __si[2];
          double __d;
        } double_convert_t;

/*}}}*/

/*{{{  constant definitions */
/*{{{  True and False */
#define TRUE 1
#define FALSE 0
/*}}}*/

/*{{{  operation and rounding codes */
#define Truncate 0 /* Conversion mode to round to zero */
#define Nearest  1 /* Conversion mode to round to nearest */

#define OpMask  3 /* Extract Rounding Mode and Operation from RMOp */
#define RMShift 2

#define Op_Add 0 /* Operation code for addition */
#define Op_Sub 1 /* Operation code for subtraction */
#define Op_Mul 2 /* Operation code for multiplication */
#define Op_Div 3 /* Operation code for division */
#define Op_Rem 4 /* Operation code for remainder, integers only */

#define RN 0 /* Round to nearest */
#define RZ 1 /* Round to zero */
#define RP 2 /* Round to +infinity */
#define RM 3 /* Round to -infinity */
/*}}}*/

/*{{{  IEEE floating point constants */
/*{{{  IEEE single length values */
#define RealShift     8
#define BitsInFrac    24
#define RealExpSh     23

#define RealExp       0X000000FF
#define RealXcess     0X0000007F
#define RealRBit      0X00000080
#define BiggestFinite 0X7F7FFFFF

#define RealInf       0X7F800000
#define RealNaN       0X7F800001

#define REAL32MININT32 0XCF000000
#define REAL32MININT64 0XDF000000
/*}}}*/

/*{{{  IEEE double length values */
#define DRealShift       11
#define DBitsInFrac      53
#define DRealExpSh       20

#define DRealExp         0X000007FF
#define DRealXcess       0X000003FF
#define DRealRBit        0X00000400
#define DBiggestFiniteLo 0XFFFFFFFF
#define DBiggestFiniteHi 0X7FEFFFFF

#define DRealInf         0X7FF00000
#define DRealNaNLo       0X00000001
#define DRealNaNHi       DRealInf

#define REAL64MININT32Hi 0XC1E00000
#define REAL64MININT64Hi 0XC2E00000
/*}}}*/

/*{{{  IEEE NaNs for 32 bit reals */
#define ZeroZeroDivNaN  0X7FC00000
#define InfInfDivNaN    0X7FA00000
#define ZeroInfMulNaN   0X7F900000
#define AddSubInfInfNaN 0X7F880000
#define NegSqrtNaN      0X7F840000
#define RemInfNaN       0X7F804000
#define RemZeroNaN      0X7F802000

#define Real64to32NaN     0X7F820000
#define Real64to32NaNfrac 0X02000000
/*}}}*/

/*{{{  IEEE NaNs for 64 bit reals */
#define DZeroZeroDivNaN  0X7FF80000
#define DInfInfDivNaN    0X7FF40000
#define DZeroInfMulNaN   0X7FF20000
#define DAddSubInfInfNaN 0X7FF10000
#define DNegSqrtNaN      0X7FF08000
#define DRemInfNaN       0X7FF00800
#define DRemZeroNaN      0X7FF00400

#define DReal64to32NaN   0X7FF04000
/*}}}*/
/*}}}*/

/*{{{  32 bit integer values */
#ifndef BitsPerWord32
#define BitsPerWord32  32
#define BytesPerWord32 4
#endif

#ifndef MOSTPOS_INT32
#define MOSTPOS_INT32 2147483647
#define MOSTNEG_INT32 ((-MOSTPOS_INT32) - 1)

#define MaxSignedInt32 MOSTPOS_INT32
#define MinSignedInt32 MOSTNEG_INT32

#define IntegerLimit32 ((MaxSignedInt32 / 10) - 1)
#endif

#ifndef MOSTPOS_INT32_UNSIGNED
#define MOSTPOS_INT32_UNSIGNED 0XFFFFFFFFU
#define MOSTNEG_INT32_UNSIGNED 0U

#define MaxUnsignedInt32 MOSTPOS_INT32_UNSIGNED
#define MinUnsignedInt32 MOSTNEG_INT32_UNSIGNED
#endif
/*}}}*/

/*{{{  32 bit word values */
#define SignBit32  0X80000000
#define LowBits32  0XFFFFFFFF
#define HighBits32 0X00000000
#define NegBits32  0X80000000
/*}}}*/

/*{{{  16 bit word values */
#define SignBit16  0X00008000
#define LowBits16  0X0000FFFF
#define HighBits16 0XFFFF0000
#define NegBits16  0XFFFF8000
/*}}}*/

/*{{{  Misc definitions used by maths functions (here for convenience) */
/* SignBit */
#define SignBit 0x80000000

/* BitsPerWord */
#define BitsPerWord (sizeof(int) * 8)
/*}}}*/

/*}}}*/

/*{{{  routine prototypes */
/*{{{  Internal real intrinsics */
#pragma IMS_translate(LONGSUM, "LONGSUM%c")
#pragma IMS_translate(LONGADD, "LONGADD%c")
#pragma IMS_translate(LONGDIFF, "LONGDIFF%c")
#pragma IMS_translate(LONGPROD, "LONGPROD%c")
#pragma IMS_translate(LONGDIV, "LONGDIV%c")
#pragma IMS_translate(SHIFTRIGHT, "ShiftRight")
#pragma IMS_translate(SHIFTLEFT, "ShiftLeft")
#pragma IMS_translate(NORMALISE, "Normalise")
#pragma IMS_translate(RealIDiv, "RealIDiv%c")
#pragma IMS_translate(r64toi64, "r64toi64%c")
#pragma IMS_translate(i64tor64, "i64tor64%c")

extern void LONGSUM(BIT32 *, BIT32 *, BIT32, BIT32, BIT32);
extern void LONGADD(BIT32 *, BIT32, BIT32, BIT32);
extern void LONGDIFF(BIT32 *, BIT32 *, BIT32, BIT32, BIT32);
extern void LONGPROD(BIT32 *, BIT32 *, BIT32, BIT32, BIT32);
extern void LONGDIV(BIT32 *, BIT32 *, BIT32, BIT32, BIT32);
extern void SHIFTRIGHT(BIT32 *, BIT32 *, BIT32, BIT32, BIT32);
extern void SHIFTLEFT(BIT32 *, BIT32 *, BIT32, BIT32, BIT32);
extern void NORMALISE(BIT32 *, BIT32 *, BIT32 *, BIT32, BIT32);
extern void RealIDiv (INT32 *Quot, INT32 *Hi, INT32 *Lo, INT32 *Guard,
                      INT32 Yfrac[2]);
extern void r64toi64(BOOL *Error, BIT32 *NHi, BIT32 *NLo, INT Mode, 
                     BIT32 XHi, BIT32 XLo);
extern void i64tor64(BIT32 *XHi, BIT32 *XLo, INT Mode, BIT32 NHi, BIT32 NLo);


#pragma IMS_nolink(LONGSUM)
#pragma IMS_nolink(LONGADD)
#pragma IMS_nolink(LONGDIFF)
#pragma IMS_nolink(LONGPROD)
#pragma IMS_nolink(LONGDIV)
#pragma IMS_nolink(SHIFTRIGHT)
#pragma IMS_nolink(SHIFTLEFT)
#pragma IMS_nolink(NORMALISE)
#pragma IMS_nolink(RealIDiv)
#pragma IMS_nolink(r64toi64)
#pragma IMS_nolink(i64tor64)


/*}}}*/

/*{{{  compiler real library */
#pragma IMS_translate(real32op,  "REAL32OPERR%")
#pragma IMS_translate(real32eq,  "REAL32EQERR%")
#pragma IMS_translate(real32gt,  "REAL32GTERR%")
#pragma IMS_translate(real64op,  "REAL64OPERR%")
#pragma IMS_translate(real64eq,  "REAL64EQERR%")
#pragma IMS_translate(real64gt,  "REAL64GTERR%")
#pragma IMS_translate(r32tor64,  "REAL32TOREAL64%")
#pragma IMS_translate(r64tor32,  "REAL64TOREAL32%")
#pragma IMS_translate(r32toi32,  "REAL32TOINT32%")
#pragma IMS_translate(i32tor32,  "INT32TOREAL32%")
#pragma IMS_translate(r64toi32,  "REAL64TOINT32%")
#pragma IMS_translate(i32tor64,  "INT32TOREAL64%")
#pragma IMS_translate(r32toui32, "REAL32TOUINT32%")
#pragma IMS_translate(r64toui32, "REAL64TOUINT32%")
#pragma IMS_translate(ui32tor32, "UINT32TOREAL32%")
#pragma IMS_translate(ui32tor64, "UINT32TOREAL64%")

extern float        real32op(float fx, int RMOp, float fy);
extern int          real32eq(float fx, float fy);
extern int          real32gt(float fx, float fy);
extern double       real64op(double *dx, int RMOp, double *dy);
extern int          real64eq(double *dx, double *dy);
extern int          real64gt(double *dx, double *dy);
extern double       r32tor64(float fx);
extern float        r64tor32(int Mode, double *dx);
extern int          r32toi32(int Mode, float fx);
extern float        i32tor32(int Mode, int N);
extern int          r64toi32(int Mode, double *dx);
extern double       i32tor64(int N);
extern unsigned int r32toui32(int Mode, float fx);
extern unsigned int r64toui32(int Mode, double *dx);
extern float        ui32tor32(int Mode, unsigned int N);
extern double       ui32tor64(unsigned int N);

#pragma IMS_nolink(real32op)
#pragma IMS_nolink(real32eq)
#pragma IMS_nolink(real32gt)
#pragma IMS_nolink(real64op)
#pragma IMS_nolink(real64eq)
#pragma IMS_nolink(real64gt)
#pragma IMS_nolink(r32tor64)
#pragma IMS_nolink(r64tor32)
#pragma IMS_nolink(r32toi32)
#pragma IMS_nolink(i32tor32)
#pragma IMS_nolink(r64toi32)
#pragma IMS_nolink(i32tor64)
#pragma IMS_nolink(r32toui32)
#pragma IMS_nolink(r64toui32)
#pragma IMS_nolink(ui32tor32)
#pragma IMS_nolink(ui32tor64)

/*}}}*/

/*{{{  compiler integer library */
#pragma IMS_translate(divide, "divide%c")
#pragma IMS_translate(udivide, "udivide%c")
#pragma IMS_translate(remainder, "remainder%c")
#pragma IMS_translate(uremainder, "uremainder%c")

int          divide(int a, int b);
unsigned int udivide(unsigned int a, unsigned int b);
int          remainder(int a, int b);
unsigned int uremainder(unsigned int a, unsigned int b);

#pragma IMS_nolink(divide)
#pragma IMS_nolink(udivide)
#pragma IMS_nolink(remainder)
#pragma IMS_nolink(uremainder)

/*}}}*/

/*}}}*/

#endif
