/* Copyright (C) SGS-THOMSON Microelectronics Ltd, 1996 */

/*
 * Monitor interface	10/25/94
 *
 * Private data structures and interfaces used in the monitor code.
 * This file is used to share declarations and such between the different
 * files implementing monitors.  It does not contain exported API.
 */

#ifndef	_BLUE_MONITOR_MD_H_
#define	_BLUE_MONITOR_MD_H_

#include "threads_md.h"
#include <semaphor.h>

/*
 * Type definitions.
 */

/* The system-level monitor data structure */
struct sys_mon {
    /* Cache support */
    int flags;

    /*
     * NOTE: SYS_MON_HAS_HANDLER and SYS_MON_IN_CACHE are preserved
     * across sysMonitorExit() and sysMonitorDestroy().  Both need to
     * be permanent properties of a monitor, which means that 
     * SYS_MON_HAS_HANDLER must never be true for a cache monitor.
     */

/*  #define SYS_MON_IN_CACHE		0x10 */

    /* Thread currently executing in this monitor */
    sys_thread_t	*monitor_owner;
    /* Number of monitors held by this thread */
    unsigned int        depth;

    /* Queue of threads trying to enter monitor */
    Semaphore   	*monitor_waitq;

    /* Queue of threads waiting while in monitor */
       sys_thread_t     *wait_list_head;
       sys_thread_t     *wait_list_tail;

};

typedef enum {
	ASYNC_REGISTER,
	ASYNC_UNREGISTER
} async_action_t;

/*
 * Macros
 */

#define SYS_MID_NULL ((sys_mon_t *) 0)

/*
 * Symbolic constant to be used when waiting indefinitly on a condition
 * variable
 */
#define SYS_TIMEOUT_INFINITY -1

typedef enum {
	SYS_ASYNC_MON_ALARM = 1,
 	SYS_ASYNC_MON_IO,
 	SYS_ASYNC_MON_EVENT,
 	SYS_ASYNC_MON_CHILD,
 	SYS_ASYNC_MON_MAX
} async_mon_key_t;

#define SYS_ASYNC_MON_INPUT SYS_ASYNC_MON_IO
#define SYS_ASYNC_MON_OUTPUT SYS_ASYNC_MON_IO

/*
 * It is legitimate to cast a sys_mon_t * to a monitor_t * so long
 * as the sys_mon_t is in a cache monitor.  However, if all you have
 * is the sys_mon_t *, you need a way to determine whether it's in
 * the cache.  We previously did that with a bounds check on the
 * monitor cache memory, as it was contiguous and never changed.
 * Now that the cache can be expanded that is no longer true, so we
 * maintain a flag in the sys_mon_t.  Currently, this is only an
 * issue when remotely killing a thread (the mon_wait monitor is
 * a sys_mon_t *).  If we move to a model where we never kill threads
 * remotely, the SYS_MON_IN_CACHE flag, mon_wait and MonInMonCache()
 * stuff can go away.
 */
#define MonInMonCache(mid) (((mid)->flags & SYS_MON_IN_CACHE) != 0)

#endif	/* _BLUE_MONITOR_MD_H_ */
