/* Copyright (C) SGS-THOMSON Microelectronics Ltd, 1996 */

/*
 * Provides stat and other related functions using the SGS-Thomson
 * Microelectronics toolset and irun.
 */

#ifndef _STAT_FIX_H_
#define _STAT_FIX_H_

#include <time.h>

typedef unsigned long ulong_t;
typedef ulong_t dev_t;                  /* expanded device type */
typedef ulong_t ino_t;                  /* expanded inode type  */
typedef ulong_t mode_t;                 /* file attribute type  */
typedef ulong_t nlink_t;                /* file link type       */
#ifndef _UID_T
#define _UID_T
typedef long    uid_t;                  /* UID type             */   
#endif
typedef uid_t   gid_t;                  /* GID type             */
typedef int     off_t;                  /* ?<offset> type */
typedef long    pid_t;                  /* process id type      */

/*   
 * Time expressed in seconds and nanoseconds
 */
        
typedef struct  timespec {              /* definition per POSIX.4 */
        time_t          tv_sec;         /* seconds */
        long            tv_nsec;        /* and nanoseconds */
} timespec_t;
 
typedef struct timespec timestruc_t;    /* definition per SVr4 */

#define  _ST_FSTYPSZ 16          /* array size for file system type name */
#define  S_ISREG(mode)       (((mode)&0xF000) == 0x8000) /*  something to do
                                                         with modes ??? */

typedef struct{
        int     dd_fd;          /* file descriptor */
        int     dd_loc;         /* offset in block */
        int     dd_size;        /* amount of valid data */
        char    *dd_buf;        /* directory block */
}DIR;                           /* stream data from opendir() */

/*
 * File-system independent directory entry.
 */
struct dirent {
        ino_t           d_ino;          /* "inode number" of entry */
        off_t           d_off;          /* offset of disk directory entry */
        unsigned short  d_reclen;       /* length of this record */
        char            d_name[1];      /* name of file */
};


/* Symbolic constants for the "access" routine: */
#define R_OK    4       /* Test for Read permission */
#define W_OK    2       /* Test for Write permission */
#define X_OK    1       /* Test for eXecute permission */
#define F_OK    0       /* Test for existence of File */

#define  S_IFDIR         0x4000  /* directory */
#define  S_IFREG         0x8000  /* regular */

struct  stat {
        dev_t   st_dev;
        long    st_pad1[3];     /* reserved for network id */
        ino_t   st_ino;
        mode_t  st_mode;
        nlink_t st_nlink;
        uid_t   st_uid;
        gid_t   st_gid;         
        dev_t   st_rdev;
        long    st_pad2[2];
        off_t   st_size;
        long    st_pad3;        /* future off_t expansion */
        timestruc_t st_atim;
        timestruc_t st_mtim;
        timestruc_t st_ctim;
        long    st_blksize;
        long    st_blocks;   
        char    st_fstype[_ST_FSTYPSZ];
        long    st_pad4[8];     /* expansion area */

};

#define st_atime        st_atim.tv_sec
#define st_mtime        st_mtim.tv_sec
#define st_ctime        st_ctim.tv_sec

extern int stat(char *, struct stat *);

#define MAXPATHLEN      1024

#endif /* !_STAT_FIX_H_ */
