/*
 * @(#)properties_md.c	1.10 95/11/29  
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/* Copyright (C) SGS-THOMSON Microelectronics Ltd, 1996 */

/*
 *  Modified for ST20 by Nigel Atkinson, July 1996
 */

#include <stdlib.h>
#include "timeval.h"
#include "sys_api.h"
#include "oobj.h"
#include "java_lang_System.h"
#include "stat_fix.h"
#include "st20misc.h"

/*
 * This routine initializes the system properties with
 * all the system defined properties.
 *
 * REMIND: We need to obtain some of these values in a more
 * organized way.
 */

#define PUTPROP(props, key, val) \
    execute_java_dynamic_method(0, (HObject *)(props), \
				"put", "(Ljava/lang/Object;Ljava/lang/Object;)Ljava/lang/Object;",\
				makeJavaString((key), strlen(key)),\
				makeJavaString((val), strlen(val)))

char **user_props = 0;

struct Hjava_util_Properties *
java_lang_System_initProperties(struct Hjava_lang_System *this, 
				struct Hjava_util_Properties *props) 
{
    char *v;
    char buf[MAXPATHLEN];	/* Needs to hold cwd */
    struct passwd *pwent;
    int uid;

    /* java properties */
    PUTPROP(props, "java.version", RELEASE);
    PUTPROP(props, "java.vendor", "Sun Microsystems Inc.");
    PUTPROP(props, "java.vendor.url", "http://www.sun.com/");
    v = getenv("JAVA_HOME");
    PUTPROP(props, "java.home", v ? v : "/usr/local/java");
    if (jio_snprintf(buf, MAXPATHLEN,
		 "%d.%d", JAVA_VERSION, JAVA_MINOR_VERSION) == -1) {
	SignalError(0, JAVAPKG "InternalError", "invalid version number");
    }
    PUTPROP(props, "java.class.version", buf);
    v = getenv("CLASSPATH");
    PUTPROP(props, "java.class.path", v ? v : "");

    /* os properties */
    PUTPROP(props, "os.name", "NO-OS");
    PUTPROP(props, "os.version", "0.0");
    PUTPROP(props, "os.arch", "ST20");

    /* file system properties */
    PUTPROP(props, "file.separator", "/");
    PUTPROP(props, "path.separator", ":");
    PUTPROP(props, "line.separator", "\n");

    /* user properties */
/*
    pwent = getpwuid(getuid());
    PUTPROP(props, "user.name", pwent ? pwent->pw_name : "?");
    PUTPROP(props, "user.home", pwent ? pwent->pw_dir : "?");
*/

    getcwd(buf, sizeof(buf));
    PUTPROP(props, "user.dir", buf);

    /* user defined properties, set by main */
    if (user_props != 0) {
	int i;
	for (i = 0 ; user_props[i] && user_props[i+1] ; i += 2) {
	    PUTPROP(props, user_props[i], user_props[i+1]);
	}
    }

    return props;
}
