/* Copyright (C) SGS-THOMSON Microelectronics Ltd, 1996 */

/* Created by Nigel Atkinson, 26/6/96 */

/*{{{  GNU copyright info */
/* Copyright (C) 1991, 1992, 1993 Free Software Foundation, Inc.
This file is part of the GNU C Library.

The GNU C Library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version.

The GNU C Library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with the GNU C Library; see the file COPYING.LIB.  If
not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA.  */
/*}}}*/

#include <errno.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <iocntrl.h>
#include "st20misc.h"
#include <limits.h>
#include <math.h>
#include <mathf.h>
#include "stat_fix.h"
#include "timeval_md.h"

int IsGetEnv(void *,void *,char *,char *);
#pragma IMS_translate (IsGetEnv, "IsGetEnv%c")
extern void *_IMS_IsFileBuffer;
extern void *_IMS_SystemLink;
static char **__environ;

/*{{{  spiel */
/* Generate a unique temporary filename using up to five characters of PFX
   if it is not NULL.  The directory to put this file in is searched for
   as follows: First the environment variable "TMPDIR" is checked.
   If it contains the name of a writable directory, that directory is used.
   If not and if DIR is not NULL, that value is checked.  If that fails,
   P_tmpdir is tried and finally "/tmp".  The storage for the filename
   is allocated by `malloc'.  */
/*}}}*/

/*{{{  tempnam */
char *
tempnam (const char *dir, const char *pfx)
{
  size_t len;
  register char *s;
  register char *t = tmpnam ((char *)NULL);

  len = strlen((const char *)t);

  if (t == NULL)
    return NULL;

  s = (char *) malloc(len);
  if (s == NULL)
    return NULL;

  (void) memcpy(s, t, len);
  return s;
}
/*}}}*/

/*{{{  strdup */
/* duplicates s1 and returns a pointer to the new string */
char *
strdup (const char *s1)
{
  char *new;
  size_t len = strlen (s1);
   
  new = (char *)malloc (len + 1);
  if (new != (char *)NULL)
    strncpy (new, s1, len + 1);
  return new;
}
/*}}}*/

/*{{{  mkdir (no modes) */
int
mkdir (const char *path, mode_t mode)
{ 
  char *cmd = (char *)malloc (80 + strlen (path));
  char *p;
  int status, save;
  struct stat statbuf;
  
  if (path == NULL){
    errno = EINVAL;
    return -1;
  }
   
  /* Check for some errors.  */
  if (stat ((char *)path, &statbuf) < 0){
    if (errno != ENOENT)
      return -1;
  /* There is no file by that name.  Good.  */
    }
  else{
    errno = EEXIST;
    return -1;
  }
  
  p = cmd;
  *p++ = 'm';
  *p++ = 'k';
  *p++ = 'd';
  *p++ = 'i'; 
  *p++ = 'r';
  *p++ = ' ';
    
  strcpy (p, path);

  save = errno;
  /* If system doesn't set errno, but the mkdir fails, we really
     have no idea what went wrong.  EIO is the vaguest error I
     can think of, so I'll use that.  */
  errno = EIO;
  status = system (cmd);
/*  if (WIFEXITED (status) && WEXITSTATUS (status) == 0){ */
    errno = save;
    return 0;
/*  }
  else
    return -1; */
}   
/*}}}*/

/*{{{  getenv */
char *getenv (const char *string)
{
  int result;
  static char val[1041]; /* space for end-of-string char, TS/1377 */
  char *tempep,*local = NULL;
  register size_t size;
  register char **ep;
  register int i;
  int localsize=0;

  if (string == NULL)
    return NULL;

  if (__environ){
    /* check local environment */
    size = strlen (string);
    for (ep = __environ; *ep != NULL; ++ep)
      if (!strncmp (*ep, string, size) && (*ep)[size] == '='){
        tempep = *ep;
        for (i = size;i>=0;(tempep)++,i--);
        localsize = strlen(tempep)+2;
        local = (char *)malloc (localsize);
        if (local == NULL)
          return tempep;
        strcpy (local,(const char *)tempep);
      }
  }
  /* check host environment */
  result = IsGetEnv(&_IMS_SystemLink, &_IMS_IsFileBuffer, (char *)string, val);
  if (result)
    /* not in host environment */
    return local;
  /* in both environments */
  if (localsize){
    memmove(&local[1],(const void *)local,localsize-1);
    local[0] = ':';
    strcat (val,(const char *)local);
  }
  return val;
}
/*}}}*/

/*{{{  putenv */
int
putenv (char *string)
{
  const char *const name_end = strchr (string, '=');
  register size_t size;
  register char **ep;
  static char *init__environ = NULL;

  __environ = &init__environ;  
  if (name_end == NULL){
    /* Remove the variable from the environment.  */
    size = strlen (string);  
    for (ep = __environ; *ep != NULL; ++ep)
      if (!strncmp (*ep, string, size) && (*ep)[size] == '='){
        while (ep[1] != NULL){
          ep[0] = ep[1];
          ++ep;
        }
        *ep = NULL;
        return 0;
      }
  }
   
  size = 0;
  for (ep = __environ; *ep != NULL; ++ep)
    if (!strncmp (*ep, string, name_end - string) && (*ep)[name_end - string] == '=')
      break;
    else
      ++size;

  if (*ep == NULL){
    static char **last_environ = NULL;
    char **new_environ = (char **) malloc ((size + 2) * sizeof (char *));
    if (new_environ == NULL)
      return -1;
    (void) memcpy ((void *) new_environ, (void *) __environ,size * sizeof (char *));
    new_environ[size] = (char *) string;
    new_environ[size + 1] = NULL;
    if (last_environ != NULL)
      free ((void *) last_environ);
    last_environ = new_environ;
    __environ = new_environ;
  }
  else   
    *ep = (char *) string;
        
  return 0;
}  
/*}}}*/

/*{{{  fstat */
int
fstat (long fd, struct stat *buf)
{
  int error = 0;

  errno = 0;
  if (fd >= 0){
    buf->st_mode    = (mode_t) 0x08000 | 0777; /* rwxrwxrwx */
    buf->st_ino     = (ino_t) 0;
    buf->st_dev     = (dev_t) 0;
    buf->st_rdev    = (dev_t) 0;
    buf->st_nlink   = (nlink_t) 0;
    buf->st_uid     = (uid_t) 100;
    buf->st_gid     = (gid_t) 100;
    buf->st_size    = (off_t) filesize ((int)fd);
    if (buf->st_size >= 0){
      buf->st_atime   = (time_t) 0;   
      buf->st_mtime   = (time_t) 0;   
      buf->st_ctime   = (time_t) 0;
      buf->st_blksize = 8192L;
      if (buf->st_size % 512)
       buf->st_blocks  = (long)(buf->st_size / 512) + 1 ;
      else
       buf->st_blocks  = (long)(buf->st_size / 512);
    }
    else
      error = -1;
  }
  else{
    error = -1; 
    errno = ENOENT;
  }

  return error;
}
/*}}}*/

/*{{{  stat */
int
stat (char *file, struct stat *buf)
{
  long int fd;
  int error = 0;
  size_t sz;
  char *regchk, *dirchk;   
      
  errno = 0;
  sz = strlen (file) + 9;
      
  regchk = (char *)malloc (sz);
  dirchk = (char *)malloc (sz);
  sprintf (regchk, "test -f %s",file);
  sprintf (dirchk, "test -d %s",file);
  
  fd = open (file, O_RDONLY | O_BINARY);
    
  if (fd >= 0){
    if (!system (regchk))
      buf->st_mode    = (mode_t) 0x08000 | 0777; /* rwxrwxrwx */
    else if (!system (dirchk))
      buf->st_mode    = (mode_t) 0x04000 | 0777; /* rwxrwxrwx */
    buf->st_ino     = (ino_t) 0;
    buf->st_dev     = (dev_t) 0;
    buf->st_rdev    = (dev_t) 0;
    buf->st_nlink   = (nlink_t) 0;
    buf->st_uid     = (uid_t) 100;
    buf->st_gid     = (gid_t) 100;
    buf->st_size    = (off_t) filesize ((int)fd);
    buf->st_atime   = (time_t) 0;
    buf->st_mtime   = (time_t) 0;
    buf->st_ctime   = (time_t) 0;
    if (buf->st_size >= 0){
      buf->st_blksize = 8192L;
      if (buf->st_size % 512)
       buf->st_blocks  = (long)(buf->st_size / 512) + 1 ;
      else
       buf->st_blocks  = (long)(buf->st_size / 512);
    }
    close ((int)fd);
  }
  else{
    errno = ENOENT;
    error = -1;
  }
      
  free (regchk);
  free (dirchk);
  return error;
}
/*}}}*/

/*{{{  pause */
int pause (void)
{
  printf ("pause() called\n");
  return 0;
}
/*}}}*/

/*{{{  gettimeofday (better) */
int
gettimeofday(struct timeval *timer, void *vd)
{
  timer->tv_sec = time(NULL);
  timer->tv_usec = 0;
  return 0;
}
/*}}}*/

/*{{{  dlerror (stub) */
char *dlerror (void)
{
  printf ("dlerror() called\n");
  return (char *)NULL;
}
/*}}}*/

/*{{{  dlopen (stub) */
void *dlopen (const char *pathname, int mode)
{
  printf ("dlopen() called\n");
  return (void *)NULL;
}
/*}}}*/

/*{{{  dlsym (stub) */
void *dlsym(void *handle, const char *name)
{
  printf ("dlsym() clled\n");
  return (void *)NULL;
}
/*}}}*/

/*{{{  getcwd (nasty) */
char *getcwd(char *buf, size_t size)
{
  buf = getenv ("PWD");
  /*printf ("getcwd: %s\n",buf);*/
  if (strlen (buf) > size)
    fprintf (stderr,"getcwd: ERROR: string too long\n");
  return buf;
}
/*}}}*/

