/*
 * @(#)system_md.c	1.18 95/11/29  
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/* Copyright (C) SGS-THOMSON Microelectronics Ltd, 1996 */

/*
 *  Modified for ST20 by Nigel Atkinson, July 1996
 */

#include <stdlib.h>
#include <time.h>
#include <stdio.h>
#include "timeval.h"
#include "sys_api.h"
#include "standardlib.h"

/* Not in sys/time.h, for some reason */
int gettimeofday(struct timeval *, void *);

long
sysGetMilliTicks()
{
    struct timeval tv;

    (void) gettimeofday(&tv, (void *) 0);
    return((tv.tv_sec * 1000) + (tv.tv_usec / 1000));
}

long
sysTime(long *l_p)
{
    return(time((time_t *)l_p));
}

int64_t
sysTimeMillis()
{
    struct timeval t;
    gettimeofday(&t, 0);
    return ll_add(ll_mul(int2ll(t.tv_sec), int2ll(1000)), 
		  int2ll(t.tv_usec/1000));
}

struct tm *
sysLocaltime(time_t *t, struct tm *tm)
{
    tm = tm;
    return localtime(t);
}

struct tm *
sysGmtime(time_t *t, struct tm *tm)
{
    tm = tm;
    return gmtime(t);
}

void
sysStrftime(char *buf, int buflen, char *fmt, struct tm *t)
{
    strftime(buf, buflen, fmt, t);
}

time_t sysMktime(struct tm *t)
{
    return mktime(t);
}

/*
 * Moved here as a portability concern - not availiable on all platforms
 *
 * The API to this is historical, and isn't exactly what we want, but
 * we can check for buffer overflow regardless.  The caller should be
 * able to guarantee against overflow, but if so, truncate silently.
 */
int jio_snprintf(char *str, size_t count, const char *fmt, ...);

void ll2str(int64_t number, char *s, char *limit)
{
#define MAGIC    10000000	/* slightly more than (2^64) ^ (1/3) */
    int group1, group2, group3;
    char *sign;
    int len = limit - s;
    const int64_t MAGIC_LL = {0,MAGIC};

  if ((signed long)number.mslong >= 0){
    sign = "";
    number = ll_neg(number); /* this lets -2^63 still work! */
  }
  else
    sign = "-";

  group3 = -( ll2int (ll_rem (number, MAGIC_LL) ) ); /* low 7 digits */
  number = ll_div ((ll_add (number, (int2ll (group3)) )), MAGIC_LL );
  group2 = -( ll2int (ll_rem (number, MAGIC_LL) ) ); /* next 7 digits */
  number = ll_div ((ll_add (number, (int2ll (group2)) )), MAGIC_LL );
  group1 = -(ll2int (number));                       /* high 7 digits */

    if (group1 != 0) {
        (void) jio_snprintf(s, len, "%s%d%07.7d%07.7d",
			    sign, group1, group2, group3);
    } else if (group2 != 0) {
        (void) jio_snprintf(s, len, "%s%d%07.7d", sign, group2, group3);
    } else {
        (void) jio_snprintf(s, len, "%s%d", sign, group3);      
    }
}

void
sysExit(int status)
{
  exit(status);
}

int
sysAtexit(void (*func)(void))
{
  return atexit(func);
}

void
sysAbort(void)
{
    abort();
}
