/*
 * Copyright (c) 1995 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/* Copyright (C) SGS-THOMSON Microelectronics Ltd, 1996 */

/*
 * threads_md.c - NO-OS implementation of Java threads
 */

#include <process.h>
#include <stdlib.h>

#include "sys_api.h"
#include "monitor.h"
#include "monitor_md.h"
#include "monitor_cache.h"
#include "threads.h"
#include "threads_md.h"

#include "java_lang_Thread.h"

/*
 * Set to TRUE once threads have been bootstrapped
 */
bool_t ThreadsInitialized = FALSE;

/*
 * Queue of active Java threads
 */
static sys_thread_t *ThreadQueue;

/*
 * Add thread to queue of active threads.
 */
static void
queueInsert(sys_thread_t *tid)
{
  QUEUE_LOCK();
  ActiveThreadCount++;
  tid->next = ThreadQueue;
  ThreadQueue = tid;
  if (!tid->system_thread)
    UserThreadCount++;
  /*printf ("ActiveThreadCount = %d, UserThreadCount = %d\n",ActiveThreadCount,UserThreadCount);*/
  QUEUE_UNLOCK();
}

/*
 * Remove thread from queue of active threads.
 */
static void
queueRemove(sys_thread_t *tid)
{
  QUEUE_LOCK();
  --ActiveThreadCount;
  if (!tid->system_thread){
    --UserThreadCount;
/* 
 * Temporarily removing this fix as it introduces another nasty bug
 * that we need to investigate and fix for 1.0.1.   --dac 1/11/96
 * (scrivener mrm)
 */
/* I have put this back in as it does not seem to cause problems --nea 7/23/96 */

	QUEUE_NOTIFY();
  }
  if (ThreadQueue == tid)
    ThreadQueue = tid->next;
  else{
    sys_thread_t *p;
    for (p = ThreadQueue; p->next != 0; p = p->next) {
      if (p->next == tid) {
        p->next = tid->next;
        break;
      }
    }
  }
  /* printf ("ActiveThreadCount = %d, UserThreadCount = %d\n",ActiveThreadCount,UserThreadCount); */
  QUEUE_UNLOCK();
}

/* 
 * Bootstrap the Java thread system by making the current thread the
 * "primordial" thread.
 */
int
sysThreadBootstrap(sys_thread_t **ptid)
{
  extern Process *_IMS_AllProc;
  extern char *_IMS_stack_base; /* Top of stack */
  extern char *_IMS_stack_limit; /* Bottom of stack */
  sys_thread_t *tid;

  /*
   * Allocate and initialize "thread private" structure.
   */
  
  if ((tid = malloc(sizeof(sys_thread_t))) == 0) {
      out_of_memory();
  }
  memset(tid, 0, sizeof(sys_thread_t));
  tid->state = RUNNABLE;
  /* tid->system_thread = TRUE; */
  tid->id = 0;
  /* add process to process list */
  tid->proc.link = _IMS_AllProc;
  _IMS_AllProc = &tid->proc;
  /* Initialise channel */
  ChanInit (&(tid->waitchan));
  tid->proc.wbase = (int*)_IMS_stack_limit;
  tid->proc.wsize = _IMS_stack_base - _IMS_stack_limit;
  tid->proc.chan  = (Channel)0x8000000;
  tid->next_waiting_thread = NULL;
  queueInsert(tid);
  
  ThreadsInitialized = TRUE;
  *ptid = tid;
  
  return SYS_OK;
}

/*
 * Return current stack pointer of specified thread.
 */
void *
sysThreadStackPointer(sys_thread_t *tid)
{
/* OUCH!!!! */

/*
    CONTEXT context;
*/
    /* REMIND: Need to fix this for Win95 */
/*
    context.ContextFlags = CONTEXT_CONTROL;
    if (!GetThreadContext(tid->handle, &context)) {
        fprintf(stderr, "GetThreadContext failed (errcode = %x)\n",
		GetLastError());
	return 0;
    }
    return (void *)context.Esp;
*/
  printf ("sysThreadStackPointer() called\n");
  return ((void *)NULL);
}

/*
 * Return stack base of specified thread.
 */
void *
sysThreadStackBase(sys_thread_t *tid)
{    
  return tid->proc.wbase;
}

/*
 * Thread start routine for new Java threads
 */
static unsigned
_start(Process *p, sys_thread_t *tid)
{
  unsigned int  regs[N_TRACED_REGS];
 
  p = p;
 /* Should thread suspend itself at this point? */
  tid->state = RUNNABLE;
  tid->start_proc(tid->start_parm);
  return 0;
}

/* 
 * Create a new Java thread. The thread is initially suspended.
 */
int
sysThreadCreate(long stack_size, unsigned int flags, void *(*proc)(void *),
		sys_thread_t **tidp, void *parm)
{
#define within_main_thread_stack_(W) \
          ( (int *)(W) >= _IMS_stack_limit && (int *)(W) <= _IMS_stack_base )
  extern int *_IMS_heap_start;
  extern int *_IMS_heap_size;
  extern int *_IMS_AllProc;
  extern int *_IMS_stack_limit;
  extern int *_IMS_stack_base;
  Process *p = (Process *)_IMS_AllProc;
  sys_thread_t *tid;
  int *ws;
  int i = 0;
  /*int heap_end = (int)_IMS_heap_start + (int)_IMS_heap_size;*/

  /* 
   * Allocate and initialize thread structure.
   */

  if ((tid = malloc(sizeof(sys_thread_t))) == 0)
    out_of_memory();
  memset(tid, 0, sizeof(sys_thread_t));
  tid->state = SUSPENDED;
  tid->start_proc = proc;
  tid->start_parm = parm;
  tid->next_waiting_thread = NULL;
  if (flags == THR_SYSTEM)
    tid->system_thread = TRUE;

  /*
   * Start the new thread.
   */

  /*printf ("The heap starts at: %x and ends at: %x\n", _IMS_heap_start, heap_end);*/

  /* this is effectively a ProcInit(), without a ProcRun() */
  /*printf ("Allocating stack for process: %ld words\n", stack_size);*/
  /* allocate stack */
  if ((ws = (int *) malloc ((size_t)stack_size)) == 0){
    out_of_memory();
  }

  ChanInit (&(tid->waitchan));

  /*printf ("Workspace address: %x\n",ws);*/

  /* check that ws is not in the main thread stack*/
  /* If _IMS_stack_limit is NULL then we are potentially called from occam
     and cannot ascertain whether we are in the main stack. Assume we are not
     and go straight to the search of the process list */   

  if (_IMS_stack_limit != NULL && within_main_thread_stack_(ws))
    printf ("ERROR: Trying to allocate process stack in main thread stack\n");

  /* search the list of processes for a workspace that includes ws */
  while (p != NULL)
  {
    i++;
    if (ws >= p->wbase && ws < (int *)((char *)p->wbase + p->wsize))
    {
     printf ("Process %d\n",i); 
     printf("auto failure - %p is between %p and %p\n",
              ws, p->wbase, (int *)((char *)p->wbase + p->wsize));
      /* ws is nested within a concurrent processes stack */
      break;
    }
    p = p->link;   
  }

  ProcInit (&tid->proc, (void (*)())_start, ws,(int)stack_size,1, tid);

  if (&tid->proc == 0){
    free(tid);
    return SYS_NORESOURCE;
  }

  queueInsert(tid);
  *tidp = tid;
  return SYS_OK;
}

/*
 * First thing called by a newly created Java thread.
 */
void
sysThreadInit(sys_thread_t *tid, stackp_t stack)
{
    /*
     * The setting of the stack base is now handled by the thread start
     * routine, so the 'stack' parameter is ignored here.
     */
}

/* 
 * Destroy calling thread (function never returns!).
 */
void
sysThreadExit(void)
{
    sys_thread_t *tid = sysThreadSelf();
    monitor_t *mon;
    TID Hthread ;
    ExecEnv *ee;

    CACHE_LOCK();
    REGISTRY_LOCK();

    queueRemove(tid);

    /*
     * Put the Java stack segments on the to-be-freed list.
     */

    Hthread = (TID)tid->cookie;
    ee = (ExecEnv *)THREAD(Hthread)->eetop;
    if (ee != 0) {  /* ee == 0 does happen... */
        DeleteExecEnv(ee, (JHandle *)Hthread);
    }

    /*
     * Notify anybody who is waiting for us to die.
     */

    mon = lookupMonitor(obj_monitor(tid->cookie));
    if (mon != 0) {
        sys_mon_t *mid = &sysmon(mon);
	sysMonitorEnter(mid);
	sysMonitorNotifyAll(mid);
	sysMonitorExit(mid);
    }

    /*
     * Clean up all monitors that we currently own.  monitorCleanup()
     * exits _moncache_lock but not _registry_lock
     */

    monitorCleanup(Hthread);
    REGISTRY_UNLOCK();
    THREAD(Hthread)->PrivateInfo = 0;

    /*
     * Free process stack, clean up process struct and free thread struct
     */

    /* disable time slicing to make sure that our **nasty** cleanup routine
       is not disturbed */   
/*
    printf ("Thread %p: disabling timeslicing\n",tid);
    __asm{ ldc 0;
           settimeslice; }
*/
    free (tid->proc.wbase);
    ProcInitClean (&tid->proc);
    free(tid);
    __asm{ stopp; }
    /* NOT REACHED */
}

/* 
 * Yield control to another thread.
 */
void
sysThreadYield(void)
{
  ProcReschedule();
}

/* 
 * Suspend execution of the specified thread.
 */
int
sysThreadSuspend(sys_thread_t *tid)
{
  if (tid->state == RUNNABLE && tid == sysThreadSelf()){
    tid->state = SUSPENDED;
    __asm{ ldlp 0;
           st tid->proc.wp;
           stopp;}
    return SYS_OK;
  }
  printf ("sysThreadSuspend: Thread not current\n");
  return SYS_ERR;
}

/* 
 * Continue execution of the specified thread.
 */
int
sysThreadResume(sys_thread_t *tid)
{
  if (tid->state == SUSPENDED){
    tid->state = RUNNABLE;
    __asm{ld tid->proc.wp;
          adc 1;
          runp;}
  }
  return SYS_OK;
}

/* 
 * Return priority of specified thread.
 */
int
sysThreadGetPriority(sys_thread_t *tid, int *pp)
{
  *pp = tid->priority;
  return SYS_OK;
}

/*
 * Set priority of specified thread.
 */
int
sysThreadSetPriority(sys_thread_t *tid, int p)
{
  tid->priority = p;
  return SYS_OK;
}

/* 
 * Return the thread information block of the calling thread.
 */
sys_thread_t *
sysThreadSelf(void)
{
  extern Process *_IMS_AllProc;
  int *wptr;
  Process *p = _IMS_AllProc;
  int found_it = 0;
  sys_thread_t *self;

  /* fudge for first thread! */
  if (!ThreadsInitialized)
    return 0;

  /* search for myself using workspace parameters and process list */

  __asm{
    ldlp 0; 
    st wptr;
  }

  /* search the list of processes for the workspace that includes the wptr */
  while (p != NULL){
    if (wptr >= p->wbase && wptr < (int *)((unsigned int)p->wbase + p->wsize)){
      /* we have found the stack for this concurrent process */
      found_it = 1;
      break;
    }
    p = p->link;
  }

  if (! found_it) {
    return 0;
  }

  /* then use macro which returns block pointer for struct */

  self = OBJECT_ADDR (sys_thread_t, proc, p);

  return self;
}

/*
 * Enumerate over all threads in active queue calling a function for
 * each one.
 */
int
sysThreadEnumerateOver(int (*func)(sys_thread_t *, void *), void *arg)
{
  sys_thread_t *tid;
  int ret = SYS_OK;
  
  QUEUE_LOCK();
  for (tid = ThreadQueue; tid != 0; tid = tid->next){
    if ((ret = (*func)(tid, arg)) != SYS_OK){
      break;
    }
  }
  QUEUE_UNLOCK();
  return ret;
}

/*
 * Helper function for sysThreadSingle()
 */
static int
threadSingleHelper(sys_thread_t *tid, void *self)
{
  return tid == self ? SYS_OK : sysThreadSuspend(tid);
  return 0;
}

/*
 * Puts each thread in the active thread queue to sleep except for the
 * calling thread. The threads must be later woken up with a corresponding
 * call to 'sysThreadMulti'. Returns SYS_OK on success, or SYS_ERR if any
 * of the threads could not be suspended.
 */
int
sysThreadSingle(void)
{
  return sysThreadEnumerateOver(threadSingleHelper, sysThreadSelf());
}

/*
 * Helper function for sysThreadMulti()
 */
static int
threadMultiHelper(sys_thread_t *tid, void *self)
{
  return tid == self ? SYS_OK : sysThreadResume(tid);
}

/*
 * Wakes up each thread in active thread queue except for the calling
 * thread. Returns SYS_ERR if not all threads could be woken up.
 */
void
sysThreadMulti(void)
{
  sysThreadEnumerateOver(threadMultiHelper, sysThreadSelf());
}

/*
 * Dump system-specific information about threads.
 */
void
sysThreadDumpInfo(sys_thread_t *tid)
{
  char *state;
  
  switch(tid->state){
    case RUNNABLE: 
    state = "RUNNABLE";
    break;
    case SUSPENDED:
    state = "SUSPENDED";
    break;
    case SLEEPING:
    state = "SLEEPING";
    break;
    case MONITOR_WAIT:
    state = "MONITOR_WAIT";
    break;
    case CONDVAR_WAIT:
    state = "CONDVAR_WAIT";
    break;
    default:
    state = "UNKNOWN";
  }
  fprintf(stderr, ", NO-OS_ID:0x%x, state:%s", tid->id, state);
}

int
sysInterruptsPending(void)
{
  return FALSE;
}

void
sysThreadSetBackPtr(sys_thread_t *t, void *p)
{
  t->cookie = p;
}

void *
sysThreadGetBackPtr(sys_thread_t *t)
{
  return t->cookie;
}

void
sysThreadInitializeSystemThreads(void)
{
  extern void InitializeFinalizerThread(void);
  /* Create the finalization thread */
  InitializeFinalizerThread();
}

int
sysThreadCheckStack(void)
{
    return 1;
}

/*
 * The mechanics of actually signalling an exception (in the future,
 * and Alarm or Interrupt) depend upon what thread implementation you
 * are using.
 */
void
sysThreadPostException(sys_thread_t *tid, void *exc)
{
    struct execenv *ee;

    ee = (ExecEnv *) THREAD((TID)tid->cookie)->eetop;
    if (ee != 0) {
        exceptionThrow(ee, exc);
    }
}

unsigned sleep(unsigned seconds)
{
    ProcWait(seconds * CLOCKS_PER_SEC_LOW);
    return 0;
}
