/*
 *  Long Long Integer Maths Library, for compilers not supporting long long ints.
 *  Written by Nigel Atkinson, SGS Thomson Microelectronics Limited.
 */
/* Copyright (C) SGS-THOMSON Microelectronics Ltd, 1996 */

#ifndef _LLMATH_H_
#define _LLMATH_H_

#include "bool.h"
#include "imsvals.h"
#include "extlib.h"

typedef unsigned int    uint_t;

#ifndef _UINT64_T
#define _UINT64_T
typedef  long long uint64_t;
#define _UINT32_T
typedef unsigned long uint32_t;
#endif

#ifndef _INT64_T
#define _INT64_T
typedef long long int64_t;
#define _UINT32_T
typedef long int32_t;
#endif

#ifdef the_st20
{{{  St20 stuff
/*
 * use these macros/functions when the compiler does not support
 * the long long type
 */

#define ll_high(a)      ((a).mslong)
#define ll_low(a)       ((a).lslong)

/* arithmetic */

int64_t ll_add          (int64_t, int64_t);
int64_t ll_sub          (int64_t, int64_t);
int64_t ll_div          (int64_t, int64_t);
int64_t ll_rem          (int64_t, int64_t);
int64_t ll_mul          (int64_t, int64_t);
#define ll_neg(a)       (ll_sub(ll_zero_const,(a)))

/* logic */

int64_t ll_not          (int64_t);
int64_t ll_and          (int64_t, int64_t);
int64_t ll_or           (int64_t, int64_t);
int64_t ll_xor          (int64_t, int64_t);
int64_t ll_shl          (int64_t, unsigned long);
int64_t ll_shr          (int64_t, unsigned long);
int64_t ll_ushr         (int64_t, unsigned long);
int64_t ll_ushl         (int64_t, unsigned long);

/* type conversions */

#define ll2int(a)       ((int)(a).lslong)
#define ll2uint(a)      ((unsigned int)(a).lslong)
int64_t uint2ll         (unsigned long);
int64_t int2ll          (long);
int64_t float2ll        (float);
float   ll2float        (int64_t);
double  ll2double       (int64_t);
int64_t double2ll       (double); 
int64_t atoll           (const char *);
int64_t a2ll            (char *);
extern void ll2str      (int64_t, char *, char *);

/* comparison operators */

#define ll_ltz(ll)      (ll_lt ((ll), ll_zero_const)))
#define ll_gez(ll)      (ll_ge ((ll), ll_zero_const)))
#define ll_eqz(a)       (ll_eq ((a), ll_zero_const))

int ll_eq               (int64_t, int64_t);
int ll_ne               (int64_t, int64_t);
int ll_ge               (int64_t, int64_t);
int ll_le               (int64_t, int64_t);
int ll_lt               (int64_t, int64_t);
int ll_gt               (int64_t, int64_t);

/* useful constant declarations */

extern const int64_t ll_zero_const;
extern const int64_t ll_one_const;
extern const int64_t ll_max_const;
extern const int64_t ll_min_const;
}}}  
#endif

/* use these macros when the compiler supports the long long type */

#define ll_high(a)      ((long)((a)>>32))
#define ll_low(a)       ((long)(a))
#define int2ll(a)       ((int64_t)(a))
#define ll2int(a)       ((int)(a))
#define ll_add(a, b)    ((a) + (b))
#define ll_and(a, b)    ((a) & (b))
#define ll_div(a, b)    ((a) / (b))
#define ll_mul(a, b)    ((a) * (b))
#define ll_neg(a)       (-(a))
#define ll_not(a)       (~(a))
#define ll_or(a, b)     ((a) | (b))
#define ll_shl(a, n)    ((a) << (n))
#define ll_shr(a, n)    ((a) >> (n))
#define ll_sub(a, b)    ((a) - (b))
#define ll_ushr(a, n)   ((unsigned long long)(a) >> (n))
#define ll_xor(a, b)    ((a) ^ (b))
#define uint2ll(a)      ((uint64_t)(unsigned long)(a))
#define ll_rem(a,b)     ((a) % (b))

#define float2ll(f)     ((int64_t) (f))
#define ll2float(a)     ((float) (a))
#define ll2double(a)    ((double) (a))
#define double2ll(f)    ((int64_t) (f))

/* comparison operators */
#define ll_ltz(ll)      ((ll)<0)
#define ll_gez(ll)      ((ll)>=0)
#define ll_eqz(a)       ((a) == 0)
#define ll_eq(a, b)     ((a) == (b))
#define ll_ne(a,b)      ((a) != (b))
#define ll_ge(a,b)      ((a) >= (b))
#define ll_le(a,b)      ((a) <= (b))
#define ll_lt(a,b)      ((a) < (b))
#define ll_gt(a,b)      ((a) > (b))

#define ll_zero_const   ((int64_t) 0)
#define ll_one_const    ((int64_t) 1)

extern void ll2str(int64_t a, char *s, char *limit);

#endif /* !_LLMATH_H_ */
