/*{{{  Header*/
/* Copyright (C) SGS-THOMSON Microelectronics Ltd, 1996 */

/*
 *  Modified for ST20 by Nigel Atkinson, July 1996
 */

/*
 * Machine Dependent implementation of the dynamic linking support
 * for java.  This routine is NO-OS specific.
 */
/*}}}  */

/*{{{  Includes*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include "sys_api.h"
#include "path_md.h"
#include "cm10misc.h"
#include "stat_fix.h"
/*}}}  */

/*{{{  Defines*/
/* from dlfcn.h */
#define RTLD_NEXT       (void *)-1      /* look in `next' shared object */
/*}}}  */

/*{{{  Globasl*/
static  int     linkerinitialized = 0;
static  int     useddlsegments = 0;
static  int     sizedlsegments = 0;
/*}}}  */

/*{{{  Prototypes*/
void out_of_memory(void);
int jio_snprintf(char *str, size_t count, const char *fmt, ...);
/*}}}  */

/*{{{  The shared libraries array*/
/* 
 * The shared libraries array 
 */
static struct dlsegment {
    char *fname;
    void *handle;
} *dlsegment = 0;
/*}}}  */

/*{{{  sysBuildLibName(char *holder, int holderlen, char *pname, char *fname)*/
/*
 * create a string for the dynamic lib open call by adding the
 * appropriate pre and extensions to a filename and the path
 */
void
sysBuildLibName(char *holder, int holderlen, char *pname, char *fname)
{
  printf ("sysBuildLibName() called\n");
}
/*}}}  */

/*{{{  sysInitializeLinker()*/
/*
 * Initialize the linker
 */
char *
sysInitializeLinker()
{
  char *ldpath;

  /*
   * setup linker search paths
   */
  if ((ldpath = getenv("LD_LIBRARY_PATH")) != 0) {
      if ((ldpath = (char *)strdup(ldpath)) == 0) {
	  out_of_memory();
      }
  }
  if (linkerinitialized) {
      return(ldpath);
  }
  dlsegment = (struct dlsegment *)
  malloc(sizeof(struct dlsegment) * (sizedlsegments = 10));
  if (dlsegment == 0) {
    out_of_memory();
  }
  dlsegment[useddlsegments].fname = "<main>";
/*
  if ((dlsegment[useddlsegments].handle = dlopen(0, 1)) == 0)
    fprintf(stderr, "Unable to dlopen main!\n");
*/
/*
  else
    useddlsegments++;
*/
  linkerinitialized = 1;
  return(ldpath);
}
/*}}}  */

/*{{{  sysAddDLSegment(char *fn)*/
/* 
 * Add a shared library to the list
 */
int
sysAddDLSegment(char *fn)
{
  printf ("sysAddDLSegment() called\n");
  return 0;
}
/*}}}  */


/*{{{  sysDynamicLink(char *sym_p)*/
long
sysDynamicLink(char *sym_p)
{
/*{{{  Function prototypes*/
void Java_java_lang_Object_getClass_stub();
void Java_java_lang_Object_hashCode_stub();
void Java_java_lang_Object_clone_stub();
void Java_java_lang_Object_notify_stub();
void Java_java_lang_Object_notifyAll_stub();
void Java_java_lang_Object_wait_stub();
void Java_java_lang_Class_forName_stub();
void Java_java_lang_Class_newInstance_stub();
void Java_java_lang_Class_getName_stub();
void Java_java_lang_Class_getSuperclass_stub();
void Java_java_lang_Class_getInterfaces_stub();
void Java_java_lang_Class_getClassLoader_stub();
void Java_java_lang_Class_isInterface_stub();
void Java_java_lang_Compiler_initialize_stub();
void Java_java_lang_Compiler_compileClass_stub();
void Java_java_lang_Compiler_compileClasses_stub();
void Java_java_lang_Compiler_command_stub();
void Java_java_lang_Compiler_enable_stub();
void Java_java_lang_Compiler_disable_stub();
void Java_java_lang_Thread_currentThread_stub();
void Java_java_lang_Thread_yield_stub();
void Java_java_lang_Thread_sleep_stub();
void Java_java_lang_Thread_start_stub();
void Java_java_lang_Thread_isAlive_stub();
void Java_java_lang_Thread_countStackFrames_stub();
void Java_java_lang_Thread_setPriority0_stub();
void Java_java_lang_Thread_stop0_stub();
void Java_java_lang_Thread_suspend0_stub();
void Java_java_lang_Thread_resume0_stub();
void Java_java_lang_Math_sin_stub();
void Java_java_lang_Math_cos_stub();
void Java_java_lang_Math_tan_stub();
void Java_java_lang_Math_asin_stub();
void Java_java_lang_Math_acos_stub();
void Java_java_lang_Math_atan_stub();
void Java_java_lang_Math_exp_stub();
void Java_java_lang_Math_log_stub();
void Java_java_lang_Math_sqrt_stub();
void Java_java_lang_Math_IEEEremainder_stub();
void Java_java_lang_Math_ceil_stub();
void Java_java_lang_Math_floor_stub();
void Java_java_lang_Math_rint_stub();
void Java_java_lang_Math_atan2_stub();
void Java_java_lang_Math_pow_stub();
void Java_java_lang_Float_toString_stub();
void Java_java_lang_Float_valueOf_stub();
void Java_java_lang_Float_floatToIntBits_stub();
void Java_java_lang_Float_intBitsToFloat_stub();
void Java_java_lang_Double_toString_stub();
void Java_java_lang_Double_valueOf_stub();
void Java_java_lang_Double_doubleToLongBits_stub();
void Java_java_lang_Double_longBitsToDouble_stub();
void Java_java_lang_System_currentTimeMillis_stub();
void Java_java_lang_System_arraycopy_stub();
void Java_java_lang_System_initProperties_stub();
void Java_java_lang_ClassLoader_defineClass_stub();
void Java_java_lang_ClassLoader_resolveClass_stub();
void Java_java_lang_ClassLoader_findSystemClass_stub();
void Java_java_lang_ClassLoader_init_stub();
void Java_java_lang_Runtime_exitInternal_stub();
void Java_java_lang_Runtime_execInternal_stub();
void Java_java_lang_Runtime_freeMemory_stub();
void Java_java_lang_Runtime_totalMemory_stub();
void Java_java_lang_Runtime_gc_stub();
void Java_java_lang_Runtime_runFinalization_stub();
void Java_java_lang_Runtime_traceInstructions_stub();
void Java_java_lang_Runtime_traceMethodCalls_stub();
void Java_java_lang_Runtime_initializeLinkerInternal_stub();
void Java_java_lang_Runtime_buildLibName_stub();
void Java_java_lang_Runtime_loadFileInternal_stub();
void Java_java_lang_SecurityManager_getClassContext_stub();
void Java_java_lang_SecurityManager_currentClassLoader_stub();
void Java_java_lang_SecurityManager_classDepth_stub();
void Java_java_lang_SecurityManager_classLoaderDepth_stub();
void Java_java_util_Date_toString_stub();
void Java_java_util_Date_toLocaleString_stub();
void Java_java_util_Date_toGMTString_stub();
void Java_java_util_Date_expand_stub();
void Java_java_util_Date_computeValue_stub();
void Java_java_io_FileDescriptor_valid_stub();
void Java_java_io_FileDescriptor_initSystemFD_stub();
void Java_java_io_FileInputStream_open_stub();
void Java_java_io_FileInputStream_read_stub();
void Java_java_io_FileInputStream_readBytes_stub();
void Java_java_io_FileInputStream_skip_stub();
void Java_java_io_FileInputStream_available_stub();
void Java_java_io_FileInputStream_close_stub();
void Java_java_io_FileOutputStream_open_stub();
void Java_java_io_FileOutputStream_write_stub();
void Java_java_io_FileOutputStream_writeBytes_stub();
void Java_java_io_FileOutputStream_close_stub();
void Java_java_io_RandomAccessFile_open_stub();
void Java_java_io_RandomAccessFile_read_stub();
void Java_java_io_RandomAccessFile_readBytes_stub();
void Java_java_io_RandomAccessFile_write_stub();
void Java_java_io_RandomAccessFile_writeBytes_stub();
void Java_java_io_RandomAccessFile_getFilePointer_stub();
void Java_java_io_RandomAccessFile_seek_stub();
void Java_java_io_RandomAccessFile_length_stub();
void Java_java_io_RandomAccessFile_close_stub();
void Java_java_io_File_exists0_stub();
void Java_java_io_File_canWrite0_stub();
void Java_java_io_File_canRead0_stub();
void Java_java_io_File_isFile0_stub();
void Java_java_io_File_isDirectory0_stub();
void Java_java_io_File_lastModified0_stub();
void Java_java_io_File_length0_stub();
void Java_java_io_File_mkdir0_stub();
void Java_java_io_File_renameTo0_stub();
void Java_java_io_File_delete0_stub();
void Java_java_io_File_list0_stub();
void Java_java_io_File_isAbsolute_stub();
void Java_java_lang_Throwable_printStackTrace0_stub();
void Java_java_lang_Throwable_fillInStackTrace_stub();
/*}}}  */

/*{{{  Lookup table entry*/
struct LUT{
    char *name;
    void (*addr)();
};
/*}}}  */

/*{{{  Function look up table*/
struct LUT FLUT[114] = {
    {"Java_java_lang_Object_getClass_stub", Java_java_lang_Object_getClass_stub},
    {"Java_java_lang_Object_hashCode_stub", Java_java_lang_Object_hashCode_stub},
    {"Java_java_lang_Object_clone_stub", Java_java_lang_Object_clone_stub},
    {"Java_java_lang_Object_notify_stub", Java_java_lang_Object_notify_stub},
    {"Java_java_lang_Object_notifyAll_stub", Java_java_lang_Object_notifyAll_stub},
    {"Java_java_lang_Object_wait_stub", Java_java_lang_Object_wait_stub},
    {"Java_java_lang_Class_forName_stub", Java_java_lang_Class_forName_stub},
    {"Java_java_lang_Class_newInstance_stub", Java_java_lang_Class_newInstance_stub},
    {"Java_java_lang_Class_getName_stub", Java_java_lang_Class_getName_stub},
    {"Java_java_lang_Class_getSuperclass_stub", Java_java_lang_Class_getSuperclass_stub},
    {"Java_java_lang_Class_getInterfaces_stub", Java_java_lang_Class_getInterfaces_stub},
    {"Java_java_lang_Class_getClassLoader_stub", Java_java_lang_Class_getClassLoader_stub},
    {"Java_java_lang_Class_isInterface_stub", Java_java_lang_Class_isInterface_stub},
    {"Java_java_lang_Compiler_initialize_stub", Java_java_lang_Compiler_initialize_stub},
    {"Java_java_lang_Compiler_compileClass_stub", Java_java_lang_Compiler_compileClass_stub},
    {"Java_java_lang_Compiler_compileClasses_stub", Java_java_lang_Compiler_compileClasses_stub},
    {"Java_java_lang_Compiler_command_stub", Java_java_lang_Compiler_command_stub},
    {"Java_java_lang_Compiler_enable_stub", Java_java_lang_Compiler_enable_stub},
    {"Java_java_lang_Compiler_disable_stub", Java_java_lang_Compiler_disable_stub},
    {"Java_java_lang_Thread_currentThread_stub", Java_java_lang_Thread_currentThread_stub},
    {"Java_java_lang_Thread_yield_stub", Java_java_lang_Thread_yield_stub},
    {"Java_java_lang_Thread_sleep_stub", Java_java_lang_Thread_sleep_stub},
    {"Java_java_lang_Thread_start_stub", Java_java_lang_Thread_start_stub},
    {"Java_java_lang_Thread_isAlive_stub", Java_java_lang_Thread_isAlive_stub},
    {"Java_java_lang_Thread_countStackFrames_stub", Java_java_lang_Thread_countStackFrames_stub},
    {"Java_java_lang_Thread_setPriority0_stub", Java_java_lang_Thread_setPriority0_stub},
    {"Java_java_lang_Thread_stop0_stub", Java_java_lang_Thread_stop0_stub},
    {"Java_java_lang_Thread_suspend0_stub", Java_java_lang_Thread_suspend0_stub},
    {"Java_java_lang_Thread_resume0_stub", Java_java_lang_Thread_resume0_stub},
    {"Java_java_lang_Math_sin_stub", Java_java_lang_Math_sin_stub},
    {"Java_java_lang_Math_cos_stub", Java_java_lang_Math_cos_stub},
    {"Java_java_lang_Math_tan_stub", Java_java_lang_Math_tan_stub},
    {"Java_java_lang_Math_asin_stub", Java_java_lang_Math_asin_stub},
    {"Java_java_lang_Math_acos_stub", Java_java_lang_Math_acos_stub},
    {"Java_java_lang_Math_atan_stub", Java_java_lang_Math_atan_stub},
    {"Java_java_lang_Math_exp_stub", Java_java_lang_Math_exp_stub},
    {"Java_java_lang_Math_log_stub", Java_java_lang_Math_log_stub},
    {"Java_java_lang_Math_sqrt_stub", Java_java_lang_Math_sqrt_stub},
    {"Java_java_lang_Math_IEEEremainder_stub", Java_java_lang_Math_IEEEremainder_stub},
    {"Java_java_lang_Math_ceil_stub", Java_java_lang_Math_ceil_stub},
    {"Java_java_lang_Math_floor_stub", Java_java_lang_Math_floor_stub},
    {"Java_java_lang_Math_rint_stub", Java_java_lang_Math_rint_stub},
    {"Java_java_lang_Math_atan2_stub", Java_java_lang_Math_atan2_stub},
    {"Java_java_lang_Math_pow_stub", Java_java_lang_Math_pow_stub},
    {"Java_java_lang_Float_toString_stub", Java_java_lang_Float_toString_stub},
    {"Java_java_lang_Float_valueOf_stub", Java_java_lang_Float_valueOf_stub},
    {"Java_java_lang_Float_floatToIntBits_stub", Java_java_lang_Float_floatToIntBits_stub},
    {"Java_java_lang_Float_intBitsToFloat_stub", Java_java_lang_Float_intBitsToFloat_stub},
    {"Java_java_lang_Double_toString_stub", Java_java_lang_Double_toString_stub},
    {"Java_java_lang_Double_valueOf_stub", Java_java_lang_Double_valueOf_stub},
    {"Java_java_lang_Double_doubleToLongBits_stub", Java_java_lang_Double_doubleToLongBits_stub},
    {"Java_java_lang_Double_longBitsToDouble_stub", Java_java_lang_Double_longBitsToDouble_stub},
    {"Java_java_lang_System_currentTimeMillis_stub", Java_java_lang_System_currentTimeMillis_stub},
    {"Java_java_lang_System_arraycopy_stub", Java_java_lang_System_arraycopy_stub},
    {"Java_java_lang_System_initProperties_stub", Java_java_lang_System_initProperties_stub},
    {"Java_java_lang_ClassLoader_defineClass_stub", Java_java_lang_ClassLoader_defineClass_stub},
    {"Java_java_lang_ClassLoader_resolveClass_stub", Java_java_lang_ClassLoader_resolveClass_stub},
    {"Java_java_lang_ClassLoader_findSystemClass_stub", Java_java_lang_ClassLoader_findSystemClass_stub},
    {"Java_java_lang_ClassLoader_init_stub", Java_java_lang_ClassLoader_init_stub},
    {"Java_java_lang_Runtime_exitInternal_stub", Java_java_lang_Runtime_exitInternal_stub},
    {"Java_java_lang_Runtime_execInternal_stub", Java_java_lang_Runtime_execInternal_stub},
    {"Java_java_lang_Runtime_freeMemory_stub", Java_java_lang_Runtime_freeMemory_stub},
    {"Java_java_lang_Runtime_totalMemory_stub", Java_java_lang_Runtime_totalMemory_stub},
    {"Java_java_lang_Runtime_gc_stub", Java_java_lang_Runtime_gc_stub},
    {"Java_java_lang_Runtime_runFinalization_stub", Java_java_lang_Runtime_runFinalization_stub},
    {"Java_java_lang_Runtime_traceInstructions_stub", Java_java_lang_Runtime_traceInstructions_stub},
    {"Java_java_lang_Runtime_traceMethodCalls_stub", Java_java_lang_Runtime_traceMethodCalls_stub},
    {"Java_java_lang_Runtime_initializeLinkerInternal_stub", Java_java_lang_Runtime_initializeLinkerInternal_stub},
    {"Java_java_lang_Runtime_buildLibName_stub", Java_java_lang_Runtime_buildLibName_stub},
    {"Java_java_lang_Runtime_loadFileInternal_stub", Java_java_lang_Runtime_loadFileInternal_stub},
    {"Java_java_lang_SecurityManager_getClassContext_stub", Java_java_lang_SecurityManager_getClassContext_stub},
    {"Java_java_lang_SecurityManager_currentClassLoader_stub", Java_java_lang_SecurityManager_currentClassLoader_stub},
    {"Java_java_lang_SecurityManager_classDepth_stub", Java_java_lang_SecurityManager_classDepth_stub},
    {"Java_java_lang_SecurityManager_classLoaderDepth_stub", Java_java_lang_SecurityManager_classLoaderDepth_stub},
    {"Java_java_util_Date_toString_stub", Java_java_util_Date_toString_stub},
    {"Java_java_util_Date_toLocaleString_stub", Java_java_util_Date_toLocaleString_stub},
    {"Java_java_util_Date_toGMTString_stub", Java_java_util_Date_toGMTString_stub},
    {"Java_java_util_Date_expand_stub", Java_java_util_Date_expand_stub},
    {"Java_java_util_Date_computeValue_stub", Java_java_util_Date_computeValue_stub},
    {"Java_java_io_FileDescriptor_valid_stub", Java_java_io_FileDescriptor_valid_stub},
    {"Java_java_io_FileDescriptor_initSystemFD_stub", Java_java_io_FileDescriptor_initSystemFD_stub},
    {"Java_java_io_FileInputStream_open_stub", Java_java_io_FileInputStream_open_stub},
    {"Java_java_io_FileInputStream_read_stub", Java_java_io_FileInputStream_read_stub},
    {"Java_java_io_FileInputStream_readBytes_stub", Java_java_io_FileInputStream_readBytes_stub},
    {"Java_java_io_FileInputStream_skip_stub", Java_java_io_FileInputStream_skip_stub},
    {"Java_java_io_FileInputStream_available_stub", Java_java_io_FileInputStream_available_stub},
    {"Java_java_io_FileInputStream_close_stub", Java_java_io_FileInputStream_close_stub},
    {"Java_java_io_FileOutputStream_open_stub", Java_java_io_FileOutputStream_open_stub},
    {"Java_java_io_FileOutputStream_write_stub", Java_java_io_FileOutputStream_write_stub},
    {"Java_java_io_FileOutputStream_writeBytes_stub", Java_java_io_FileOutputStream_writeBytes_stub},
    {"Java_java_io_FileOutputStream_close_stub", Java_java_io_FileOutputStream_close_stub},
    {"Java_java_io_RandomAccessFile_open_stub", Java_java_io_RandomAccessFile_open_stub},
    {"Java_java_io_RandomAccessFile_read_stub", Java_java_io_RandomAccessFile_read_stub},
    {"Java_java_io_RandomAccessFile_readBytes_stub", Java_java_io_RandomAccessFile_readBytes_stub},
    {"Java_java_io_RandomAccessFile_write_stub", Java_java_io_RandomAccessFile_write_stub},
    {"Java_java_io_RandomAccessFile_writeBytes_stub", Java_java_io_RandomAccessFile_writeBytes_stub},
    {"Java_java_io_RandomAccessFile_getFilePointer_stub", Java_java_io_RandomAccessFile_getFilePointer_stub},
    {"Java_java_io_RandomAccessFile_seek_stub", Java_java_io_RandomAccessFile_seek_stub},
    {"Java_java_io_RandomAccessFile_length_stub", Java_java_io_RandomAccessFile_length_stub},
    {"Java_java_io_RandomAccessFile_close_stub", Java_java_io_RandomAccessFile_close_stub},
    {"Java_java_io_File_exists0_stub", Java_java_io_File_exists0_stub},
    {"Java_java_io_File_canWrite0_stub", Java_java_io_File_canWrite0_stub},
    {"Java_java_io_File_canRead0_stub", Java_java_io_File_canRead0_stub},
    {"Java_java_io_File_isFile0_stub", Java_java_io_File_isFile0_stub},
    {"Java_java_io_File_isDirectory0_stub", Java_java_io_File_isDirectory0_stub},
    {"Java_java_io_File_lastModified0_stub", Java_java_io_File_lastModified0_stub},
    {"Java_java_io_File_length0_stub", Java_java_io_File_length0_stub},
    {"Java_java_io_File_mkdir0_stub", Java_java_io_File_mkdir0_stub},
    {"Java_java_io_File_renameTo0_stub", Java_java_io_File_renameTo0_stub},
    {"Java_java_io_File_delete0_stub", Java_java_io_File_delete0_stub},
    {"Java_java_io_File_list0_stub", Java_java_io_File_list0_stub},
    {"Java_java_io_File_isAbsolute_stub", Java_java_io_File_isAbsolute_stub},
    {"Java_java_lang_Throwable_printStackTrace0_stub", Java_java_lang_Throwable_printStackTrace0_stub},
    {"Java_java_lang_Throwable_fillInStackTrace_stub", Java_java_lang_Throwable_fillInStackTrace_stub},
};
/*}}}  */

  int i;

  if (!linkerinitialized) {
    (void)sysInitializeLinker();
  }

  for (i=0;i<(sizeof(FLUT) / sizeof(struct LUT)); i++){
    if (strcmp (sym_p,FLUT[i].name) == 0)
      return (long)FLUT[i].addr;
  }

  printf ("sysDynamicLink (%s)\n",sym_p);
  return 0;
}
/*}}}  */
