/*
 * @(#)multicast.c	1.5 95/12/02 Pavani Diwanji
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

#include <errno.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/systeminfo.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <arpa/nameser.h>
#include <resolv.h>
#include <stdio.h>

#include "java_net_InetAddress.h"
#include "sun_net_MulticastSocket.h"
#include "java_io_FileDescriptor.h"


#define JAVANETPKG 	"java/net/"

/* Multicast Socket classes */
/* These classes should be part of java.net not sun.net - and when 
this happens, move the code below to socket.c */

void sun_net_MulticastSocket_multicastJoin(struct Hsun_net_MulticastSocket *this, struct Hjava_net_InetAddress *mcast_address)
{
  struct sockaddr_in name;
  struct ip_mreq mname;
  Classjava_net_InetAddress *addrptr;
  Classsun_net_MulticastSocket *thisptr = unhand(this);
  Classjava_io_FileDescriptor *fdptr = unhand(thisptr->fd);

  if (fdptr == 0) {
    SignalError(0, JAVAPKG "NullPointerException", "null FileDescriptor");
    return;
  }

  if (mcast_address == 0) {
      SignalError(0, JAVAPKG "NullPointerException", 0);
      return;
  }

  addrptr = unhand(mcast_address);  

  /* Set the multicast group address in the ip_mreq field */
  /* eventually this check should be done by the security manager */
  mname.imr_multiaddr.s_addr = htonl(addrptr->address);
  if (!IN_MULTICAST(mname.imr_multiaddr.s_addr)) {
        struct execenv *ee = EE();
        if (!(ee && exceptionOccurred(ee))) {
            SignalError(0, JAVANETPKG "SocketException", strerror(errno));
        }
  }
  /* Set the interface to be default */
  mname.imr_interface.s_addr = INADDR_ANY;
  
  /* Join the multicast group */
  if (setsockopt(fdptr->fd- 1, IPPROTO_IP, IP_ADD_MEMBERSHIP, (char *) &mname,
	   sizeof (mname)) < 0) {
        struct execenv *ee = EE();
        if (!(ee && exceptionOccurred(ee))) {
            SignalError(0, JAVANETPKG "SocketException", strerror(errno));
        }
  }
  return;
}

void sun_net_MulticastSocket_multicastLeave(struct Hsun_net_MulticastSocket *this, struct Hjava_net_InetAddress *mcast_address)
{
  struct sockaddr_in name;
  struct ip_mreq mname;
  Classjava_net_InetAddress *addrptr;
  Classsun_net_MulticastSocket *thisptr = unhand(this);
  Classjava_io_FileDescriptor *fdptr = unhand(thisptr->fd);

  if (fdptr == 0) {
    SignalError(0, JAVAPKG "NullPointerException", "null FileDescriptor");
    return;
  }

  if (mcast_address == 0) {
      SignalError(0, JAVAPKG "NullPointerException", 0);
      return;
  }

  addrptr = unhand(mcast_address);  

  /* Set the multicast group address in the ip_mreq field */
  /* eventually this check should be done by the security manager */
  mname.imr_multiaddr.s_addr = htonl(addrptr->address);
  if (!IN_MULTICAST(mname.imr_multiaddr.s_addr)) {
        struct execenv *ee = EE();
        if (!(ee && exceptionOccurred(ee))) {
            SignalError(0, JAVANETPKG "SocketException", strerror(errno));
        }
  }
  /* Set the interface to be default */
  mname.imr_interface.s_addr = INADDR_ANY;
  
  /* Join the multicast group */
  if (setsockopt(fdptr->fd- 1, IPPROTO_IP, IP_DROP_MEMBERSHIP,(char *) &mname,
	   sizeof (mname)) < 0) {
        struct execenv *ee = EE();
        if (!(ee && exceptionOccurred(ee))) {
            SignalError(0, JAVANETPKG "SocketException", strerror(errno));
        }
  }
  return;
}

void sun_net_MulticastSocket_setTTL(struct Hsun_net_MulticastSocket *this, 
char ttl)
{
    Classsun_net_MulticastSocket *thisptr = unhand(this);
    Classjava_io_FileDescriptor *fdptr = unhand(thisptr->fd);

    if (fdptr == 0) {
      SignalError(0, JAVAPKG "NullPointerException", "null FileDescriptor");
      return;
    }

    /* setsockopt to be correct ttl */
    if (setsockopt(fdptr->fd -1, IPPROTO_IP, IP_MULTICAST_TTL, (char*)&ttl,
	    sizeof (ttl)) < 0) {
	struct execenv *ee = EE();
	if (!(ee && exceptionOccurred(ee))) 
	    SignalError(0, JAVANETPKG "SocketException", strerror(errno));
    }    
}






