/*{{{  Header*/
/*
 * @(#)path_md.c        1.18 95/10/18
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/* Copyright (C) SGS-THOMSON Microelectronics Ltd, 1996 */

/*
 *  Modified for ST20 by Nigel Atkinson, July 1996
 */

/*
 * Machine dependent path name and file name manipulation code
 */
/*}}}  */

/*{{{  Includes*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include "stat_fix.h"
#include "cm10misc.h"
#include "path.h"       /* for cpe_t */
#include "zip.h"        /* for zip_t */
/*}}}  */


cpe_t **
sysGetClassPath(void)
{
    static cpe_t **classpath;

    if (classpath == 0) {
	char *cps, *s;
	int ncpe = 1;
	cpe_t **cpp;
	/*printf("Unable to get static classpath\n"); This is NULL on first call anyhow!*/
	if ((cps = getenv("CLASSPATH")) == 0) {
	    printf("Unable to get env CLASSPATH\n");
	    cps = ".:classes";
	}
	if ((cps = strdup(cps)) == 0) {
	    return 0;
	}
	for (s = cps; *s != '\0'; s++) {
	    if (*s == ':') {
		ncpe++;
	    }
	}
	cpp = classpath = malloc((ncpe + 1) * sizeof(cpe_t *));
	if (cpp == 0) {
	    return 0;
	}
	while (cps && *cps) {
	    struct stat sbuf;
	    char *path = cps;
	    if ((cps = strchr(cps, ':')) != 0) {
		*cps++ = '\0';
	    }
	    if (*path == '\0') {
		path = ".";
	    }
	    if (sysStat(path, &sbuf) == 0 && S_ISREG(sbuf.st_mode)) {
		zip_t *zip = zip_open(path);
		if (zip != 0) {
		    cpe_t *cpe = malloc(sizeof(cpe_t));
		    if (cpe == 0) {
			return 0;
		    }
		    cpe->type = CPE_ZIP;
		    cpe->u.zip = zip;
		    *cpp++ = cpe;
		}
	    } else {
		cpe_t *cpe = malloc(sizeof(cpe_t));
		if (cpe == 0) {
		    return 0;
		}
		cpe->type = CPE_DIR;
		cpe->u.dir = path;
		*cpp++ = cpe;
	    }
	}
	*cpp = 0;
    }
    return classpath;
}
