/*{{{  Header*/
/* Copyright (C) SGS-THOMSON Microelectronics Ltd, 1996 */

/*
 * Copyright (c) 1995 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/*}}}  */

/*
 * threads_md.c - cm10 implementation of Java threads
 */

/*{{{  Includes*/
/* STREAM nan kernel threading stuff */
#include <exec/chExec.h>
#include <sync/chSem.h>

#include <stdlib.h>

#include "sys_api.h"
#include "monitor.h"
#include "monitor_md.h"
#include "monitor_cache.h"
#include "threads.h"
#include "threads_md.h"

#include "java_lang_Thread.h"
/*}}}  */

/*{{{  Globals*/
/*
 * Set to TRUE once threads have been bootstrapped
 */
bool_t ThreadsInitialized = FALSE;

/*
 * Queue of active Java threads
 */
static sys_thread_t *ThreadQueue;

/* Locking mechanism for queue interrogation for sysThreadSelf */

/* No longer using this James. */
/* well, I'm using them again, cos things keep blowing up */


bool_t __queue_sem_initialised=FALSE;
KnSem __queue_semaphore;
/*}}}  */

/*{{{  queueInsert()*/
/*
 * Add thread to queue of active threads.
 */
static void
queueInsert(sys_thread_t *tid)
{
  QUEUE_LOCK();

  /*printf("Locking my queue semaphore\n");*/

  /*semP(&__queue_semaphore, K_NOTIMEOUT);     My own lock for the threadSelf() fn */

  ActiveThreadCount++;
  tid->next = ThreadQueue;
  ThreadQueue = tid;
  if (!tid->system_thread)
    UserThreadCount++;
  /*printf ("ActiveThreadCount = %d, UserThreadCount = %d\n",ActiveThreadCount,UserThreadCount);*/

  /*printf("Unlocking my queue semaphore\n");
  semV(&__queue_semaphore);*/

  QUEUE_UNLOCK();
}
/*}}}  */
/*{{{  queueRemove()*/
/*
 * Remove thread from queue of active threads.
 */
static void
queueRemove(sys_thread_t *tid)
{
  QUEUE_LOCK();
  /*printf("Locking my queue semaphore\n");*/
  /*semP(&__queue_semaphore, K_NOTIMEOUT);     My own lock for the threadSelf() fn */
  --ActiveThreadCount;
  if (!tid->system_thread){
    --UserThreadCount;
/* 
 * Temporarily removing this fix as it introduces another nasty bug
 * that we need to investigate and fix for 1.0.1.   --dac 1/11/96
 * (scrivener mrm)
 */
/* I have put this back in as it does not seem to cause problems --nea 7/23/96 */

        QUEUE_NOTIFY();
  }
  if (ThreadQueue == tid)
    ThreadQueue = tid->next;
  else{
    sys_thread_t *p;
    for (p = ThreadQueue; p->next != 0; p = p->next) {
      if (p->next == tid) {
        p->next = tid->next;
        break;
      }
    }
  }
  /* printf ("ActiveThreadCount = %d, UserThreadCount = %d\n",ActiveThreadCount,UserThreadCount);*/
  /*printf("Unlocking my queue semaphore\n");
  semV(&__queue_semaphore);*/
  QUEUE_UNLOCK();
}
/*}}}  */
/*{{{  sysThreadBootstrap()*/
/* 
 * Bootstrap the Java thread system by making the current thread the
 * "primordial" thread.
 * AKA the first thread.
 * This thread is the root thread -- don't need to create one -- forge
 * the entry in the thread list for this one.
 */
int
sysThreadBootstrap(sys_thread_t **ptid)
{
  /*{{{  Vars*/
  extern int main(int, char **);
  extern stackp_t mainstktop;
  sys_thread_t *tid;
  /*}}}  */



  /*{{{  Allocate and initialize "thread private" structure.*/
  /*
   * Allocate and initialize "thread private" structure.
   */

  if ((tid = calloc(1,sizeof(sys_thread_t))) == 0) {
      out_of_memory();
  }


  tid->state = RUNNABLE;

  /* Give this thread an ID -- get thread id */
  tid->id  = threadSelf();

  /*printf("JAMES: Root thread id=%i\n",tid->id);*/

  /* system specific stuff*/

  tid->start_info.dsType=K_DEFAULT_START_INFO;
  tid->start_info.dsSvStackSize=K_DEFAULT_STACK_SIZE;
  tid->start_info.dsEntry=(KnPc) main;                      /* Denote the root thread */
  tid->start_info.dsUserStackPointer=mainstktop;
  tid->start_info.dsPrivilege=K_USERTHREAD;

  tid->suspensions = 0;
  semInit(&tid->wait,0);     /* Conditional variable */

  /*{{{  ST20stuff*/
  #ifdef st20_again
  /* tid->system_thread = TRUE; */
  /* add process to process list */
  tid->proc.link = _IMS_AllProc;
  _IMS_AllProc = &tid->proc;

  /* Initialise channel */
  ChanInit (&(tid->waitchan));
  tid->proc.wbase = (int*)_IMS_stack_limit;
  tid->proc.wsize = _IMS_stack_base - _IMS_stack_limit;
  tid->proc.chan  = (Channel)0x8000000;
  tid->next_waiting_thread = NULL; /* Not sure waht to do with this */
  #endif
  /*}}}  */

  /*}}}  */

  /*printf("Adding this bootsrtapped thread to the queue\n");*/

  queueInsert(tid);

  /*printf("queueInsert() done\n");*/

  ThreadsInitialized = TRUE;
  *ptid = tid;

  return SYS_OK;
}
/*}}}  */
/*{{{  sysThreadStackPointer()*/
/*
 * Return current stack pointer of specified thread.
 */
void *
sysThreadStackPointer(sys_thread_t *tid)
{
/* OUCH!!!! */

/*
    CONTEXT context;
*/
    /* REMIND: Need to fix this for Win95 */
/*
    context.ContextFlags = CONTEXT_CONTROL;
    if (!GetThreadContext(tid->handle, &context)) {
        fprintf(stderr, "GetThreadContext failed (errcode = %x)\n",
                GetLastError());
        return 0;
    }
    return (void *)context.Esp;
*/
  /*printf ("sysThreadStackPointer() called\n");*/
  return ((void *)NULL);
}

/*}}}  */
/*{{{  sysThreadStackBase()*/
/*
 * Return stack base of specified thread.
 */
void *
sysThreadStackBase(sys_thread_t *tid)
{    
  printf("In sysThreadStackBase\n");
  return tid->start_info.dsUserStackPointer;
}

/*}}}  */
/*{{{  _start()*/
/*
 * Thread start routine for new Java threads
 */
static unsigned
_start(sys_thread_t *tid)
{
  unsigned int  regs[N_TRACED_REGS];

 printf("_start() called for thread %i\n", tid->id);

 /* Should thread suspend itself at this point? */
  tid->state = RUNNABLE;
  tid->start_proc(tid->start_parm);
  return 0;
}
/*}}}  */
/*{{{  sysThreadCreate()*/
/* 
 * Create a new Java thread. The thread is initially suspended.
 */
int
sysThreadCreate(long stack_size, unsigned int flags, void *(*proc)(void *),
                sys_thread_t **tidp, void *parm)
{
  /*{{{  Vars*/
  sys_thread_t *tid;
  int result;
  char * new_stack=(char *)memalign((size_t)4,stack_size);
  void *start_ptr;
  KnThreadCtx md;  /* Mark Debbage spots another wicked problem!
                      I was starting the thread with a 'garbage' parameter
                      to teh _start() fn. pointer.
                      God only knows how it actually ran before!
                    */
  /*}}}  */

  /*{{{  Allocate and initialize thread structure.*/
  /* 
   * Allocate and initialize thread structure.
   */

  /*{{{  Alloc*/
  tid = (sys_thread_t *)calloc(1,sizeof(sys_thread_t));
  if (tid == NULL)
    out_of_memory();


    /* check allocated stack */
    if (new_stack == 0){
      out_of_memory();
    }

  /*}}}  */
  start_ptr=(void *)&(tid->start_info);

  tid->start_info.dsType=K_DEFAULT_START_INFO;
  tid->start_info.dsSvStackSize=K_DEFAULT_STACK_SIZE;
  tid->start_info.dsEntry=(KnPc) _start;
  tid->start_info.dsUserStackPointer=(void *)(new_stack+stack_size);
  tid->start_info.dsPrivilege=K_USERTHREAD;

  tid->state = SUSPENDED;
  tid->start_proc = proc;
  tid->start_parm = parm;
  tid->next_waiting_thread = NULL;
  if (flags == THR_SYSTEM)
    tid->system_thread = TRUE;

  tid->suspensions = 1;      /* Threads are created in suspended mode */
  semInit(&tid->wait,0);     /* Conditional variable */

  /*}}}  */

  /*
   * Start the new thread.
   */

  result= threadCreate(
                        K_MYACTOR,    /* Use this process to thread from */
                        (KnThreadLid *) &(tid->id),   /* Just uint32 */
                        K_SUSPENDED,  /* Start in Suspended mode */
                        NULL,         /* Inherit scheduling stuff */
                        (void*)start_ptr     /* Info on thread to start it */
                      );

  /*{{{  Check result*/
  if (result < 0){
    free(tid);
    return SYS_NORESOURCE;
  }
  /*}}}  */

  result=threadContext(K_MYACTOR, tid->id, K_CURRCTX, &md, NULL);
  /*{{{  Check result*/
  if (result < 0){
    free(tid);
    return SYS_ERR;
  }
  /*}}}  */
  md.local[1]=(KnChamReg) tid;
  result=threadContext(K_MYACTOR, tid->id, K_CURRCTX, NULL,&md);
  /*{{{  Check result*/
  if (result < 0){
    free(tid);
    return SYS_ERR;
  }
  /*}}}  */

  queueInsert(tid);
  *tidp = tid;

  printf("Created thread with id %i\n",tid->id);

  return SYS_OK;
}
/*}}}  */
/*{{{  sysThreadInit()*/
/*
 * First thing called by a newly created Java thread.
 */
void
sysThreadInit(sys_thread_t *tid, stackp_t stack)
{
    /*
     * The setting of the stack base is now handled by the thread start
     * routine, so the 'stack' parameter is ignored here.
     */
}
/*}}}  */
/*{{{  sysThreadExit()*/
/* 
 * Destroy calling thread (function never returns!).
 */
void
sysThreadExit(void)
{
    sys_thread_t *tid = sysThreadSelf();
    monitor_t *mon;
    TID Hthread ;
    ExecEnv *ee;

    CACHE_LOCK();
    REGISTRY_LOCK();

    queueRemove(tid);

    /*
     * Put the Java stack segments on the to-be-freed list.
     */

    Hthread = (TID)tid->cookie;
    ee = (ExecEnv *)THREAD(Hthread)->eetop;
    if (ee != 0) {  /* ee == 0 does happen... */
        DeleteExecEnv(ee, (JHandle *)Hthread);
    }

    /*
     * Notify anybody who is waiting for us to die.
     */

    mon = lookupMonitor(obj_monitor(tid->cookie));
    if (mon != 0) {
        sys_mon_t *mid = &sysmon(mon);
        sysMonitorEnter(mid);
        sysMonitorNotifyAll(mid);
        sysMonitorExit(mid);
    }

    /*
     * Clean up all monitors that we currently own.  monitorCleanup()
     * exits _moncache_lock but not _registry_lock
     */

    monitorCleanup(Hthread);
    REGISTRY_UNLOCK();
    THREAD(Hthread)->PrivateInfo = 0;

    /*
     * Free process stack, clean up process struct and free thread struct
     */

    printf("\n\nNuking  thread with id %i\n\n\n",tid->id);

    free(tid);
    /* Not sure if this is what they mean but -- */

    if (K_MYSELF!=tid->id)
      {
        printf("James, you are in trouble mate!\n");
      }
    threadDelete(K_MYACTOR,K_MYSELF);

    /* NOT REACHED */

    /* Well, it's not reached by this thread anyhow! */
}
/*}}}  */
/*{{{  sysThreadYield()*/
/* 
 * Yield control to another thread.
 */

/* Arrgh assuming we have pre-emptive scheduling (not normal at the moment) */
/* So this lot requires a high priority process to kick eveyone off now and again */

void
sysThreadYield(void)
{
  threadDelay(K_NOBLOCK);
}

/*}}}  */
/*{{{  sysThreadSuspend()*/
/* 
 * Suspend execution of the specified thread.
 */
int
sysThreadSuspend(sys_thread_t *tid)
{
  printf("sysThreadSuspend called\n");
  if (K_MYSELF==tid->id)
    {
      printf("Suspending myself -- problem no?\n");
    }
  if (threadSuspend(K_MYACTOR,tid->id)==K_OK)
    {
      tid->suspensions=tid->suspensions+1;
      return SYS_OK;
    }
  else
    return SYS_ERR;
}
/*}}}  */
/*{{{  sysThreadResume()*/
/* 
 * Continue execution of the specified thread.
 */
int
sysThreadResume(sys_thread_t *tid)
{
    int result;

    printf("sysThreadResume() Called on thread %i\n",tid->id);
    printf("This thread has %i suspensions, and will now have them removed\n",
            tid->suspensions);

    /* Decrement thread's suspend count until no longer suspended */
    tid->state = RUNNABLE;
    while (tid->suspensions>0)
      {
        result=threadResume(K_MYACTOR,tid->id);
        if (result!=K_OK) return SYS_ERR;
        tid->suspensions=tid->suspensions-1;
      }

   printf("sysThreadResume() complete\n");
   return SYS_OK;
}
/*}}}  */
/*{{{  sysThreadGetPriority()*/
/* 
 * Return priority of specified thread.
 */
int
sysThreadGetPriority(sys_thread_t *tid, int *pp)
{
  *pp = tid->priority;
  return SYS_OK;
}
/*}}}  */
/*{{{  sysThreadSetPriority()*/
/*
 * Set priority of specified thread.
 */
int
sysThreadSetPriority(sys_thread_t *tid, int p)
{
  tid->priority = p;
  printf("James: put threadScheduler() call to change the priority\n");
  printf("New priority for thread %i is %i\n", tid->id, p);
  return SYS_OK;
}
/*}}}  */
/*{{{  sysThreadSelf()*/
/* 
 * Return the thread information block of the calling thread.
 */
sys_thread_t *
sysThreadSelf(void)
{
  /*{{{  Vars*/
  sys_thread_t *self=NULL, *walk;
  int counter,MyThreadId=threadSelf();
  /*}}}  */
  /*{{{  Initialise queue searching semaphore*/
  if (__queue_sem_initialised==FALSE)
    {
      semInit(&__queue_semaphore,1);
      __queue_sem_initialised=TRUE;
    }
  /*}}}  */


  /* How do I do this ?*/
  /* Rummage through the threadqueue right? looking for myself?*/
  if (ThreadsInitialized == FALSE) return 0;

  /*printf("Locking my queue semaphore\n");*/
  semP(&__queue_semaphore, K_NOTIMEOUT);
 /* printf("Yo G!\n");*/

  /* OK, so it's rude to lock the queue for this little thing, you do it then!*/
  /*QUEUE_LOCK(); -- right idea -- using a semaphore now, this just made a
  recursive call to this function == BOOM!
  */

  walk=ThreadQueue;

  while(self==NULL)
    {
      if (walk->id==MyThreadId)
       {
         /*printf("Found thread id %i\n",walk->id);*/
         self=walk;
       }
      walk=walk->next;
    }


  /*QUEUE_UNLOCK();*/

  /*printf("Yeah G!\n");*/
  /*printf("Unlocking my queue semaphore\n");*/
  semV(&__queue_semaphore);

  return self;

}
/*}}}  */
/*{{{  sysThreadEnumerateOver()*/
/*
 * Enumerate over all threads in active queue calling a function for
 * each one.
 */
int
sysThreadEnumerateOver(int (*func)(sys_thread_t *, void *), void *arg)
{
  sys_thread_t *tid;
  int ret = SYS_OK;

  QUEUE_LOCK();
  for (tid = ThreadQueue; tid != 0; tid = tid->next){
    if ((ret = (*func)(tid, arg)) != SYS_OK){
      break;
    }
  }
  QUEUE_UNLOCK();
  return ret;
}
/*}}}  */
/*{{{  threadSingleHelper()*/
/*
 * Helper function for sysThreadSingle()
 */
static int
threadSingleHelper(sys_thread_t *tid, void *self)
{
  return tid == self ? SYS_OK : sysThreadSuspend(tid);
  return 0;
}
/*}}}  */
/*{{{  sysThreadSingle()*/
/*
 * Puts each thread in the active thread queue to sleep except for the
 * calling thread. The threads must be later woken up with a corresponding
 * call to 'sysThreadMulti'. Returns SYS_OK on success, or SYS_ERR if any
 * of the threads could not be suspended.
 */
int
sysThreadSingle(void)
{
  return sysThreadEnumerateOver(threadSingleHelper, sysThreadSelf());
}
/*}}}  */
/*{{{  threadMultiHelper()*/
/*
 * Helper function for sysThreadMulti()
 */
static int
threadMultiHelper(sys_thread_t *tid, void *self)
{
  return tid == self ? SYS_OK : sysThreadResume(tid);
}
/*}}}  */
/*{{{  sysThreadMulti()*/
/*
 * Wakes up each thread in active thread queue except for the calling
 * thread. Returns SYS_ERR if not all threads could be woken up.
 */
void
sysThreadMulti(void)
{
  sysThreadEnumerateOver(threadMultiHelper, sysThreadSelf());
}
/*}}}  */
/*{{{  sysThreadDumpInfo()*/
/*
 * Dump system-specific information about threads.
 */
void
sysThreadDumpInfo(sys_thread_t *tid)
{
  char *state;

  switch(tid->state){
    case RUNNABLE: 
    state = "RUNNABLE";
    break;
    case SUSPENDED:
    state = "SUSPENDED";
    break;
    case SLEEPING:
    state = "SLEEPING";
    break;
    case MONITOR_WAIT:
    state = "MONITOR_WAIT";
    break;
    case CONDVAR_WAIT:
    state = "CONDVAR_WAIT";
    break;
    default:
    state = "UNKNOWN";
  }
  fprintf(stderr, ", C0_thread_id:0x%x, state:%s", tid->id, state);
}
/*}}}  */
/*{{{  sysInterruptsPending()*/
int
sysInterruptsPending(void)
{
  return FALSE;
}
/*}}}  */
/*{{{  sysThreadSetBackPtr()*/
void
sysThreadSetBackPtr(sys_thread_t *t, void *p)
{
  t->cookie = p;
}
/*}}}  */
/*{{{  sysThreadGetBackPtr()*/
void *
sysThreadGetBackPtr(sys_thread_t *t)
{
  return t->cookie;
}
/*}}}  */
/*{{{  sysThreadInitializeSystemThreads()*/
void
sysThreadInitializeSystemThreads(void)
{
  extern void InitializeFinalizerThread(void);
  /* Create the finalization thread */
  InitializeFinalizerThread();
  printf("OK, back from InitializeFinalizerThread in  sysThreadInitializeSystemThreads\n");
}
/*}}}  */
/*{{{  sysThreadCheckStack()*/
int
sysThreadCheckStack(void)
{
    return 1;
}
/*}}}  */
/*{{{  sysThreadPostException()*/
/*
 * The mechanics of actually signalling an exception (in the future,
 * and Alarm or Interrupt) depend upon what thread implementation you
 * are using.
 */
void
sysThreadPostException(sys_thread_t *tid, void *exc)
{
    struct execenv *ee;

    ee = (ExecEnv *) THREAD((TID)tid->cookie)->eetop;
    if (ee != 0) {
        exceptionThrow(ee, exc);
    }
}
/*}}}  */
/*{{{  sleep()*/
unsigned sleep(unsigned seconds)
{
    KnTimeVal wait;

    wait.tmSec=seconds;
    wait.tmNSec=0;

    threadDelay(&wait);

    return 0;
}
/*}}}  */
