/*{{{  Header*/
/*
 * @(#)compiler.c       1.4 95/11/29  
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */
/*}}}  */

/*{{{  Includes*/
#include "oobj.h"
#include "interpreter.h"
#include "exceptions.h"
#include "java_lang_Compiler.h"
#include "java_lang_String.h"
#include "java_lang_Object.h"
/*}}}  */

/*{{{  Globals*/
static int JavaVersion = (JAVA_VERSION << 16) | JAVA_MINOR_VERSION;

/* The name of the attribute that represents compiled code on this machine. */
char *CompiledCodeAttribute = NULL;
/*}}}  */

/*{{{  Prototypes*/
static void (*p_InitializeForCompiler)(ClassClass *cb) = 0;
static void (*p_CompilerFreeClass)(ClassClass *cb) = 0;
static bool_t (*p_invokeCompiledMethod)(JHandle *o, struct methodblock *mb, 
			       int args_size, ExecEnv *ee) = 0; 
static void (*p_CompiledCodeSignalHandler)(int sig, void *info, void *uc) = 0;

static bool_t (*p_CompilerCompileClass)(ClassClass *cb) = 0;
static bool_t (*p_CompilercompileClasses)(Hjava_lang_String *name) = 0;
static Hjava_lang_Object *(*p_CompilerCommand)(Hjava_lang_Object *any) = 0;
static void (*p_CompilerEnable)() = 0;
static void (*p_CompilerDisable)() = 0;
/*}}}  */

/*{{{  void InitializeForCompiler()*/
void InitializeForCompiler(ClassClass *cb) { 
    if (p_InitializeForCompiler != NULL) 
	p_InitializeForCompiler(cb);
}
/*}}}  */
/*{{{  void CompilerFreeClass()*/
void CompilerFreeClass(ClassClass *cb) { 
    if (p_CompilerFreeClass != NULL) 
	p_CompilerFreeClass(cb);
}
/*}}}  */
/*{{{  void CompilerCompileClass()*/
void CompilerCompileClass(ClassClass *cb) { 
    if (p_CompilerCompileClass != NULL) 
	p_CompilerCompileClass(cb);
}
/*}}}  */
/*{{{  invokeCompiledMethod()*/
bool_t 
invokeCompiledMethod(JHandle *o, struct methodblock *mb, 
		     int args_size, ExecEnv *ee) 
{
    if (p_invokeCompiledMethod == NULL) { 
	SignalError(ee, JAVAPKG "InternalError", 
		"Error! Compiled methods not supported");
	return FALSE;
    } else { 
	return p_invokeCompiledMethod(o, mb, args_size, ee);
    }
}
/*}}}  */
/*{{{  void CompiledCodeSignalHandler()*/
void CompiledCodeSignalHandler(int sig, void *info, void *uc) { 
    if (p_CompiledCodeSignalHandler != NULL) 
	p_CompiledCodeSignalHandler(sig, info, uc);
}
/*}}}  */

/*{{{  Struct*/
extern struct StrIDhash *interfaceHash;
/*}}}  */
/*{{{  Function pointer table*/
void* CompiledCodeLinkVector[] = {
	(void *)&JavaVersion,

	(void *)&p_InitializeForCompiler,
	(void *)&p_invokeCompiledMethod,
	(void *)&p_CompiledCodeSignalHandler,
	(void *)&p_CompilerFreeClass,
	(void *)&p_CompilerCompileClass,
	(void *)&p_CompilercompileClasses,
	(void *)&p_CompilerEnable,
	(void *)&p_CompilerDisable,

	(void *)&CompiledCodeAttribute,
	(void *)&UseLosslessQuickOpcodes,

	(void *)&malloc,
	(void *)&calloc,
	(void *)&realloc,
	(void *)&free,

	(void *)&binclasses,
	(void *)&nbinclasses,
	(void *)&lock_classes,
	(void *)&unlock_classes,

	(void *)&classJavaLangClass,
	(void *)&classJavaLangObject,
	(void *)&classJavaLangString,
	(void *)&classJavaLangThrowable,
	(void *)&classJavaLangException,
	(void *)&classJavaLangRuntimeException,

	(void *)&EE,
	(void *)&SignalError,
	(void *)&exceptionInternalObject,

	(void *)&GetClassConstantClassName,
	(void *)&ResolveClassConstant,
	(void *)&ResolveClassConstantFromClass,
	(void *)&VerifyClassAccess,
	(void *)&FindClassFromClass,
	(void *)&dynoLink,
	(void *)&do_execute_java_method_vararg,
	(void *)&invokeJavaMethod,
	(void *)&invokeSynchronizedJavaMethod,

	(void *)&monitorEnter,
	(void *)&monitorExit,
	(void *)&monitorRegister,
	(void *)&sysMonitorSizeof,
	(void *)&sysMonitorEnter,
	(void *)&sysMonitorExit,

	(void *)&ObjAlloc,
	(void *)&ArrayAlloc,
	(void *)&MultiArrayAlloc,
	(void *)&sizearray,
	(void *)&newobject,
	(void *)&is_instance_of,
	(void *)&classname2string,
	(void *)&ID2Str,
	(void *)&interfaceHash,

	(void *)&DumpThreads,
};
/*}}}  */

/*{{{  void java_lang_Compiler_initialize ()*/
void java_lang_Compiler_initialize (Hjava_lang_Compiler *this) 
{
    void *address = (void *)sysDynamicLink("java_lang_Compiler_start");
    if (address != 0) {
	 (*(void (*) (void **)) address) (CompiledCodeLinkVector);
    }
}
/*}}}  */
/*{{{  java_lang_Compiler_compileClass()*/
long
java_lang_Compiler_compileClass(Hjava_lang_Compiler *this, 
				     Hjava_lang_Class *clazz) { 
    if (clazz == NULL) { 
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return FALSE;
    } else if (p_CompilerCompileClass != NULL) { 
	return p_CompilerCompileClass(unhand(clazz));
    } else { 
	return FALSE;
    }
}
/*}}}  */
/*{{{  java_lang_Compiler_compileClasses()*/
long
java_lang_Compiler_compileClasses(Hjava_lang_Compiler *this, 
				     Hjava_lang_String *name) { 
    if (name == NULL) { 
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return FALSE;
    } else if (p_CompilercompileClasses != NULL) { 
	return p_CompilercompileClasses(name);
    } else { 
	return FALSE;
    }
}
/*}}}  */
/*{{{  java_lang_Compiler_command()*/
Hjava_lang_Object *
java_lang_Compiler_command(Hjava_lang_Compiler *this, 
			   Hjava_lang_Object *x) { 
    if (x == NULL) { 
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return NULL;
    } else if (p_CompilerCommand != NULL) { 
	return p_CompilerCommand(x);
    } else { 
	return NULL;
    }
}
/*}}}  */
/*{{{  java_lang_Compiler_enable()*/
void 
java_lang_Compiler_enable(Hjava_lang_Compiler *this) { 
    if (p_CompilerEnable != NULL) 
	p_CompilerEnable();
}
/*}}}  */
/*{{{  java_lang_Compiler_disable()*/
void 
java_lang_Compiler_disable(Hjava_lang_Compiler *this) { 
    if (p_CompilerDisable != NULL) 
	p_CompilerDisable();
}
/*}}}  */
