/*
 * @(#)exception.c      1.26 95/12/06  
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/*-
 *      Implementation of class Throwable
 *
 *      former classruntime.c, Wed Jun 26 18:43:20 1991
 */

#include <stdio.h>
#include <signal.h>

#include "tree.h"
#include "javaString.h"

#include "monitor.h"
#include "threads.h"
#include "exceptions.h"

#include "java_lang_Throwable.h"
#include "java_lang_Thread.h"
#include "java_io_PrintStream.h"

#undef min
#define min(x, y) (((int)(x) < (int)(y)) ? (x) : (y))

extern HArrayOfByte *MakeByteString(char *str, long len);

void
java_lang_Throwable_printStackTrace0(Hjava_lang_Throwable *o,
				     Hjava_io_PrintStream *s)
{
    struct HArrayOfInt *backtrace = (HArrayOfInt *)(unhand(o)->backtrace);
    if (backtrace != NULL) {
	long *real_data = unhand(backtrace)->body;
	unsigned char **data = (unsigned char **)(real_data);
	unsigned char **enddata = 
	     (unsigned char **)(real_data + obj_length(backtrace));
	for (; data < enddata; data++) { 
	    if (*data != 0) {
		#define LEN 128
		char where[LEN];
		int i = 0;
		HArrayOfByte *bytes;

		strncpy(where + i, "\tat ", 4); i += 4;
		pc2string(*data, 0, where + i, where + sizeof(where));
		i = strlen(where);
		i = min(i, LEN-2);
		strncpy(where + i, "\n\0", 2); i += 1;  /* don't include the null */

		bytes = MakeByteString(where, i);
		execute_java_dynamic_method(EE(), (HObject*)s,
					    "write", "([B)V", bytes);
	    }
	}
    }
}

/* The pc's are saved as an int[] (to make it easy for the GC).  The
 * following constant indicates how may array elements are needed per pc
 * It will be 1 on 32-bit machines and 2 on 64-bit machines.
 */
#define MULTIPLIER (sizeof(char *)/sizeof(long))

void 
fillInStackTrace(Hjava_lang_Throwable *o, ExecEnv *ee) {
    int         size;
    Classjava_lang_Throwable *obj = unhand(o);
    JavaFrame    *start_frame = ee->current_frame;
    struct HArrayOfInt *backtrace = (HArrayOfInt *)obj->backtrace;
    JavaFrame   *frame;
    long *real_data;
    unsigned char **data, **enddata;

    /* Go backwards, removing all frames that are just part of the <init> of
     * of the throwable object "o"
     */
    while ((start_frame != NULL) && 
	   (start_frame->current_method != NULL) && 
	   !strcmp(fieldname(&start_frame->current_method->fb), "<init>") && 
	   (start_frame->vars[0].h == (JHandle *)o))
	start_frame = start_frame->prev;

    /* how many useful frames are there on the stack? */
    for (size = 0, frame = start_frame; frame != NULL; frame = frame->prev)
	if (frame->current_method != NULL) size++;

    /* if backtrace is null or not long enough, increase it */
    if ((backtrace == NULL) || obj_length(backtrace) < size * MULTIPLIER) {
	backtrace = (HArrayOfInt *)ArrayAlloc(T_INT, size * MULTIPLIER);
	if (backtrace == NULL) 
	    return;
	obj->backtrace = (HObject *)backtrace;
    }

    /* Fill in the backtrace array with the pc's */
    real_data = unhand(backtrace)->body;
    data = (unsigned char **)(real_data);
    enddata = (unsigned char **)(real_data + obj_length(backtrace));
    for (frame = start_frame; 
	     frame != NULL && data < enddata; /* second test "just in case" */
	     frame = frame->prev) {
	if (frame->current_method != NULL)
	    *data++ = frame->lastpc;
    } 
    /* Zero out any remaining space. */
    while (data < enddata) 
	*data++ = 0;
}


/*
 * Fill in the current stack trace in this exception.  This is
 * usually called automatically when the exception is created but it
 * may also be called explicitly by the user.  This routine returns
 * `this' so you can write 'throw e.fillInStackTrace();'
 */
Hjava_lang_Throwable *
java_lang_Throwable_fillInStackTrace(Hjava_lang_Throwable *obj) {
    ExecEnv *ee = (ExecEnv *)(THREAD(JthreadSelf())->eetop);

    fillInStackTrace(obj, ee);
    return obj;
}

