/*
 * @(#)SwitchData.java	1.9 95/12/20 Arthur van Hoff
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package sun.tools.asm;

import sun.tools.java.*;
import java.util.Hashtable;
import java.util.Enumeration;

public final
class SwitchData {
    int minValue, maxValue;
    Label defaultLabel = new Label();
    Hashtable tab = new Hashtable();

    /**
     * Get a label
     */
    public Label get(int n) {
	return (Label)tab.get(new Integer(n));
    }
    
    /**
     * Get a label
     */
    public Label get(Integer n) {
	return (Label)tab.get(n);
    }

    /**
     * Add a label
     */
    public void add(int n, Label lbl) {
	if (tab.size() == 0) {
	    minValue = n;
	    maxValue = n;
	} else {
	    if (n < minValue) {
		minValue = n;
	    }
	    if (n > maxValue) {
		maxValue = n;
	    }
	}
	tab.put(new Integer(n), lbl);
    }

    /**
     * Get the default label
     */
    public Label getDefaultLabel() {
	return defaultLabel;
    }

    /**
     * Return the keys of this enumaration sorted in ascending order 
     */
    public synchronized Enumeration sortedKeys() {
	return new SwitchDataEnumeration(tab);
    }
}

class SwitchDataEnumeration implements Enumeration { 
    private Integer table[];
    private int current_index = 0;

    /**
     * Create a new enumeration from the hashtable.  Each key in the
     * hash table will be an Integer, with the value being a label.  The
     * enumeration returns the keys in sorted order.
     */
    SwitchDataEnumeration(Hashtable tab) { 
	table = new Integer[tab.size()];
	int i = 0;
	for (Enumeration e = tab.keys() ; e.hasMoreElements() ; ) {
	    table[i++] = (Integer)e.nextElement();
	}
	sortTable();
	current_index = 0;
    }

    /**
     * Are there more keys to return?
     */
    public boolean hasMoreElements() { 
	return current_index < table.length;
    }

    /**
     * Return the next key.
     */
    public Object nextElement() { 
	return table[current_index++];
    }

    /**
     * Sort the table used by the enumeration.  We use a bubble sort.  It is
     * slow, but easy to code.  There aren't usually lots of keys in a
     * switch.
     */
    private void sortTable() {
	Integer c[] = this.table;
	boolean done;
	do {
	    done = true;
	    for (int i = table.length - 1 ; i > 0 ; i--) {
		if (c[i-1].intValue() > c[i].intValue()) { 
		    Integer tmp = c[i];
		    c[i] = c[i-1];
		    c[i-1] = tmp;
		    done = false;
		}
	    }
	} while (!done);
    }
}



