/*
 * @(#)clock.h	1.8 95/11/29  
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/*
 * External interface to the clock manager (generic to Green threads)
 */

#ifndef _GREEN_CLOCK_H_
#define _GREEN_CLOCK_H_

#include "timeval_md.h"
#include "threads_md.h"
#include "monitor_md.h"

/*
 * Operations on timevals.
 *
 * NB: timerCmp does not work for >=, <= or ==.
 */

#define timerIsSet(tvp)         ((tvp)->tv_sec || (tvp)->tv_usec)
#define timerCmp(tvp, uvp, cmp) \
        ((tvp)->tv_sec cmp (uvp)->tv_sec || \
         (tvp)->tv_sec == (uvp)->tv_sec && (tvp)->tv_usec cmp (uvp)->tv_usec)
#define timerEq(tvp, uvp) \
         ((tvp)->tv_sec == (uvp)->tv_sec && (tvp)->tv_usec == (uvp)->tv_usec)
#define timerClear(tvp)         (tvp)->tv_sec = (tvp)->tv_usec = 0

void timerAdd(timeval_t*, timeval_t*);
void timerSub(timeval_t*, timeval_t*);
void threadGetTimeout(timeval_t *, int*, int*);
void threadSetTimeout(timeval_t *, int, int);
/*
 * Bootstrap
 */
void clockInit(void);

/*
 * Cleanup 
 */
void cleanupPendingAlarm(sys_thread_t *);

/*
 * Monitor timeout registery
 */
void registerAlarm(sys_thread_t *, timeval_t);

/*
 * Initialize clock
 */
void InitializeClock(void);

#endif /* !_GREEN_CLOCK_H_ */
