/*
 * @(#)context_md.h	1.16 95/11/13  
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/*
 * Solaris-dependent API
 */

#ifndef _SOLARIS_CONTEXT_MD_H_
#define _SOLARIS_CONTEXT_MD_H_

#include <errno.h>
#include <ucontext.h>
#include <sys/regset.h>
#ifdef i386
#include <signal.h>
#endif

/*
 * Routines that call getcontext have strange control flow graphs,
 * since a call to a routine that calls setcontext will eventually
 * return at the getcontext site, not the original call site.  This
 * utterly wrecks control flow analysis.
 */
#pragma unknown_control_flow(getcontext)

#ifdef i386
#include <signal.h>
#endif

/*
 * A solaris2 context.  Everything else is on the stack.
 */
typedef struct {
#ifdef i386
    /* hack to fix the problem in get/set context system call
     * we save the value in edi reg here before getting
     * getcontext(), and restore this value during setcontext()
     */
    unsigned int edi;
    sigset_t oldMask, blockMask;
#endif
    unsigned int unix_errno;
    ucontext_t ucontext;
} context_t;

/* Save state of the current content and call rescheduler */

void reschedule(void);

#if i386

int  getedi(void);
void setedi(int);

#define yieldContext(contextp) \
    sigfillset(&((contextp)->blockMask)); \
    sigprocmask(SIG_BLOCK, (const sigset_t *)&((contextp)->blockMask), \
		&((contextp)->oldMask)); \
    (contextp)->edi = getedi(); \
    if (! getcontext(&((contextp)->ucontext))) { \
	sigprocmask(SIG_SETMASK, \
		    (const sigset_t *)&((contextp)->oldMask), NULL); \
	((contextp)->ucontext).uc_mcontext.gregs[EDI] = (contextp)->edi; \
	(contextp)->unix_errno = errno; \
	reschedule(); \
    } else { \
	sigprocmask(SIG_SETMASK, &((contextp)->oldMask), NULL); \
	setedi((contextp)->edi); \
    }

#else

#define yieldContext(contextp) \
    if (! getcontext(&((contextp)->ucontext))) { \
	(contextp)->unix_errno = errno; \
	reschedule(); \
    }

#endif /* i386 */

/* switch to the new context */

#ifdef i386
#define switchContext(contextp) { \
    errno = (contextp)->unix_errno; \
    ((contextp)->ucontext).uc_mcontext.gregs[EAX] = 1; \
    setcontext(&((contextp)->ucontext)); \
}
#else
#define switchContext(contextp) { \
    errno = (contextp)->unix_errno; \
    ((contextp)->ucontext).uc_mcontext.gregs[11] = 1; \
    setcontext(&((contextp)->ucontext)); \
}
#endif /* i386 */

#endif /* !_SOLARIS_CONTEXT_MD_H_ */
