/*
 * @(#)monitor_md.h	1.25 95/12/03
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/*
 * Monitor interface	10/25/94
 *
 * Private data structures and interfaces used in the monitor code.
 * This file is used to share declarations and such between the different
 * files implementing monitors.  It does not contain exported API.
 */

#ifndef	_GREEN_MONITOR_MD_H_
#define	_GREEN_MONITOR_MD_H_

#include "threads_md.h"

/*
 * Type definitions.
 */

/* The system-level monitor data structure */
struct sys_mon {
    /* Cache support */
    struct sys_mon	*pendingq;	/* Queue of interrupt info */

    short		entry_count;	/* Recursion depth */
    short		flags;		/* Various flags */

    /*
     * NOTE: SYS_MON_HAS_HANDLER and SYS_MON_IN_CACHE are preserved
     * across sysMonitorExit() and sysMonitorDestroy().  Both need to
     * be permanent properties of a monitor, which means that 
     * SYS_MON_HAS_HANDLER must never be true for a cache monitor.
     */

#define SYS_MON_STICKY_NOTIFICATION	0x1
#define SYS_MON_PENDING_NOTIFICATION	0x2
#define SYS_MON_INVERTED_PRIORITY	0x4
#define SYS_MON_HAS_HANDLER		0x8 /* Interrupt handler registered */
#define SYS_MON_IN_CACHE		0x10

    /* Thread currently executing in this monitor */
    sys_thread_t	*monitor_owner;

    /* Queue of threads trying to enter monitor */
    sys_thread_t	*monitor_waitq;

    /* Queue of threads suspended while in monitor */
    sys_thread_t	*suspend_waitq;

    /*
     * Condition Variable, otherwise know as the queue of threads
     * currently executing "wait()" inside a monitor
     */
    sys_thread_t	*condvar_waitq;

    sys_mon_t		*next_inversion;
    int			inversionPriority;
};

typedef enum {
	ASYNC_REGISTER,
	ASYNC_UNREGISTER
} async_action_t;

/*
 * Macros
 */

#define SYS_MID_NULL ((sys_mon_t *) 0)

/*
 * Symbolic constant to be used when waiting indefinitly on a condition
 * variable
 */
#define SYS_TIMEOUT_INFINITY -1

typedef enum {
	SYS_ASYNC_MON_ALARM = 1,
 	SYS_ASYNC_MON_IO,
 	SYS_ASYNC_MON_EVENT,
 	SYS_ASYNC_MON_CHILD,
 	SYS_ASYNC_MON_MAX
} async_mon_key_t;

#define SYS_ASYNC_MON_INPUT SYS_ASYNC_MON_IO
#define SYS_ASYNC_MON_OUTPUT SYS_ASYNC_MON_IO

/*
 * It is legitimate to cast a sys_mon_t * to a monitor_t * so long
 * as the sys_mon_t is in a cache monitor.  However, if all you have
 * is the sys_mon_t *, you need a way to determine whether it's in
 * the cache.  We previously did that with a bounds check on the
 * monitor cache memory, as it was contiguous and never changed.
 * Now that the cache can be expanded that is no longer true, so we
 * maintain a flag in the sys_mon_t.  Currently, this is only an
 * issue when remotely killing a thread (the mon_wait monitor is
 * a sys_mon_t *).  If we move to a model where we never kill threads
 * remotely, the SYS_MON_IN_CACHE flag, mon_wait and MonInMonCache()
 * stuff can go away.
 */
#define MonInMonCache(mid) (((mid)->flags & SYS_MON_IN_CACHE) != 0)

void resumeMonitor(sys_thread_t *tid);

/*
 * Support for the async part of async GC
 */
extern sys_mon_t *PendingNotifyQ;
#define SYS_INTERRUPTS_PENDING() (PendingNotifyQ)

void monitorApplyInversion(sys_mon_t *, sys_thread_t *);
sys_mon_t *asyncMon(async_mon_key_t);
int asyncNotifier(async_action_t action, async_mon_key_t key);

#endif	/* _GREEN_MONITOR_MD_H_ */
