/*
 * @(#)process_md.c	1.18 95/11/29
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

#include <stdlib.h>
#include <fcntl.h>
#include <standardlib.h>
#include <sys/types.h>
#include <wait.h>
#include <sys/signal.h>
#include <errno.h>
 
#include "oobj.h"
#include "tree.h"
#include "interpreter.h"
#include "javaString.h"
#include "typecodes.h"
#include "sys_api.h"
#include "monitor.h"
#include "monitor_md.h"
#include "iomgr.h"

#include "java_io_FileDescriptor.h"
#include "java_io_FileInputStream.h"
#include "java_lang_System.h"
#include "java_lang_Runtime.h"
#include "java_lang_Process.h"
#include "java_lang_UNIXProcess.h"
#include "java_lang_ProcessReaper.h"
 
/*
 * This routine is called once from the Process Reaper thread and
 * never returns.  It's job is to catch sigchld events and pass them
 * on to class UNIXProcess.
 */
void
java_lang_ProcessReaper_waitForDeath(Hjava_lang_ProcessReaper *this) 
{
    siginfo_t info;
    sys_mon_t *mon = asyncMon(SYS_ASYNC_MON_CHILD);
    char *classname = JAVAPKG "UNIXProcess";
    ClassClass *cb = FindClass(0, classname, TRUE);

    if (cb == NULL) {
	SignalError(0, JAVAPKG "NoClassDefFoundError", classname);
	return;
    }

    while (1) {
	sysMonitorEnter(mon);
	while (waitid(P_ALL, 0, &info, WEXITED|WNOHANG) != 0 || info.si_pid == 0) {
	    sysMonitorWait(mon, TIMEOUT_INFINITY);
	}
	fprintf(stderr, "pid %d status %d\n", info.si_pid, info.si_status);
	execute_java_static_method(EE(), cb, "deadChild", "(II)V",
				   info.si_pid, info.si_status);
	sysMonitorExit(mon);
    }
}


long
java_lang_UNIXProcess_waitForUNIXProcess(Hjava_lang_UNIXProcess *this)
{
    siginfo_t info;

    waitid(P_PID, unhand(this)->pid, &info, WEXITED);
    return 1;
}


long
java_lang_UNIXProcess_fork(Hjava_lang_UNIXProcess *this) 
{
    int pid = 0;
    int fdin[2], fdout[2], fderr[2], fdsync[2];
    Classjava_lang_UNIXProcess *thisptr = unhand(this);
    Classjava_io_FileDescriptor *infdptr = unhand(thisptr->stdin_fd);
    Classjava_io_FileDescriptor *outfdptr = unhand(thisptr->stdout_fd);
    Classjava_io_FileDescriptor *errfdptr = unhand(thisptr->stderr_fd);
    Classjava_io_FileDescriptor *syncfdptr = unhand(thisptr->sync_fd);

    pipe(fdin);  /* stdin */
    pipe(fdout);  /* stdout */
    pipe(fderr);  /* stderr */
    pipe(fdsync);  /* fd for synchronization */

/*    sysThreadSingle(); */
    if ((pid = fork()) == 0) {
	/* Child process */
	dup2(fdin[1], 0);
	dup2(fdout[1], 1);
	dup2(fderr[1], 2);

	close(fdin[0]);
	close(fdin[1]);
	close(fdout[0]);
	close(fdout[1]);
	close(fderr[0]);
	close(fderr[1]);
	{ int err;
	    char c;
	    err = (*systable[SYS_READ].addr)(fdsync[1], &c, 1);
	    fprintf(stderr, "err = %d!\n", err);
	}
    } else {
	/* parent process */
/*	sysThreadMulti(); */
	infdptr->fd = fdin[0]+1;
	outfdptr->fd = fdout[0]+1;
	errfdptr->fd = fderr[0]+1;
	syncfdptr->fd = fdsync[0]+1;

	close(fdin[1]);
	close(fdout[1]);
	close(fderr[1]);
	close(fdsync[1]);
    }
    return pid;
}


void
java_lang_UNIXProcess_exec(Hjava_lang_UNIXProcess *this, HArrayOfString *cmdarray,
HArrayOfString *envp) 
{
char **envstr;
#define MAXCMDARGS 198			/* why this number? */
    char *cmdstr[MAXCMDARGS+2];
    int len;

    len = obj_length(cmdarray);
    if (len == 0 || len > MAXCMDARGS) {
	SignalError(0, JAVAPKG "IllegalArgumentException", 0);
	return;
    }
#define BODYOF(h)   unhand(h)->body
    cmdstr[len] = 0;
    while (len--) {
	cmdstr[len] = allocCString(BODYOF(cmdarray)[len]);
    }

    if (envp) len = obj_length (envp);
    if (!envp || len == 0) {
 	execv(cmdstr[0], cmdstr);
     } else {
	int i;

 	envstr = (char **)calloc((len+1), sizeof(char *));
 	if (envstr == 0) {
 	    SignalError(0, JAVAPKG "IllegalArgumentException", 0);
 	    return;
 	}
 	envstr[len] = 0;
 	for (i=0; i<len; ++i) {
 	    envstr[i] = makeCString(BODYOF(envp)[i]);
 	}
 	execve(cmdstr[0], cmdstr, envstr);
     }

    fprintf(stderr, "exec of \"%s\" failed with errno %d\n", cmdstr[0], errno);
    sysAbort();
}




Hjava_lang_Process *
java_lang_Runtime_execInternal(Hjava_lang_Runtime *this, 
			       HArrayOfString *cmdarray, HArrayOfString *envp) 
{
    Hjava_lang_Process *proc = 0;

    if (cmdarray == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return 0;
    }
	
    /*
	We need a way to know which subclass of Process we want to create.
    */

    proc = (Hjava_lang_Process *) execute_java_constructor(EE(),
				  JAVAPKG "UNIXProcess",
				  (ClassClass *) 0, 
				  "([Ljava/lang/String;[Ljava/lang/String;)", 
				  cmdarray, envp);
    return proc;
}
 
void
java_lang_UNIXProcess_destroy(Hjava_lang_UNIXProcess *this)
{
    kill(unhand(this)->pid, SIGTERM);
}
