/*
 * @(#)linker_md.c	1.14 95/12/03  
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/*
 * Machine Dependent implementation of the dynamic linking support
 * for java.  This routine is Solaris specific.
 */

#include <stdio.h>
#include <dlfcn.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>

#include "sys_api.h"
#include "path_md.h"

static	int	linkerinitialized = 0;
static	int	useddlsegments = 0;
static	int	sizedlsegments = 0;

void out_of_memory(void);
int jio_snprintf(char *str, size_t count, const char *fmt, ...);

/* 
 * The shared libraries array 
 */
static struct dlsegment {
    char *fname;
    void *handle;
} *dlsegment = 0;

/*
 * create a string for the dynamic lib open call by adding the
 * appropriate pre and extensions to a filename and the path
 */
void
sysBuildLibName(char *holder, int holderlen, char *pname, char *fname)
{
    /* Quietly truncate on buffer overflow.  Should be an error. */
    (void) jio_snprintf(holder, holderlen, "%s/lib%s.so", pname, fname);
    return;
}

/*
 * Initialize the linker
 */
char *
sysInitializeLinker()
{
    char *ldpath;

    /*
     * setup linker search paths
     */
    if ((ldpath = getenv("LD_LIBRARY_PATH")) != 0) {
	if ((ldpath = strdup(ldpath)) == 0) {
	    out_of_memory();
	}
    }
    if (linkerinitialized) {
	return(ldpath);
    }
    dlsegment = (struct dlsegment *)
	malloc(sizeof(struct dlsegment) * (sizedlsegments = 10));
    if (dlsegment == 0) {
	out_of_memory();
    }
    dlsegment[useddlsegments].fname = "<main>";
    if ((dlsegment[useddlsegments].handle = dlopen(0, 1)) == 0)
	fprintf(stderr, "Unable to dlopen main!\n");
    else
	useddlsegments++;
    linkerinitialized = 1;
    return(ldpath);
}

/* 
 * Add a shared library to the list
 */
int
sysAddDLSegment(char *fn)
{
    int    i;

#ifdef DEBUG
    strcpy(fn + strlen(fn) - 3, "_g.so");
#endif
    for (i = 0; i < useddlsegments; i++) {
	if (strcmp(dlsegment[i].fname, fn) == 0) {
	    return 1;
	}
    }

    if (access(fn, R_OK) != 0) {
	return 0;
    }

    if (useddlsegments >= sizedlsegments) {
	sizedlsegments += 10;
	dlsegment = (struct dlsegment *)
	    realloc(dlsegment, sizeof(struct dlsegment) * sizedlsegments);
	if (dlsegment == 0) {
	    out_of_memory();
	}
    }

    if ((dlsegment[useddlsegments].handle = dlopen(fn, 1)) == 0) {
	char *err = (char *) dlerror();
	if (err) {
	    fprintf(stderr, "%s (%s)\n", err, fn);
	}
	return 0;
    }

    dlsegment[useddlsegments].fname = strdup(fn);
    if (dlsegment[useddlsegments].fname == 0) {
	out_of_memory();
    }
    useddlsegments++;
    return 1;
}


long
sysDynamicLink(char *sym_p)
{
    void *f = 0;
    int    i;

    /*
     * we need this kludge, because the java interpreter internals need
     * to access this before the Linker class is loaded and calls
     * the static initializers
     */
    if (!linkerinitialized) {
	(void)sysInitializeLinker();
    }
    
    if ((f = dlsym(RTLD_NEXT, sym_p)) == 0) {
	for (i = useddlsegments; --i >= 0;) {
	    if ((f = dlsym(dlsegment[i].handle, sym_p))) {
		return((long)f);
	    }
	}
    }
    return 0;
}

