/*
 * @(#)security.c	1.10 95/11/29 Jonathan Payne
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

#include "native.h"
#include "java_lang_SecurityManager.h"
#include "java_lang_ClassLoader.h"

HArrayOfObject *
java_lang_SecurityManager_getClassContext(Hjava_lang_SecurityManager *this) 
{
    struct javaframe *frame;
    HArrayOfObject *ctx;
    HObject **ctxdata;
    int n = 0;

    /* count the classes on the execution stack */
    for (frame = EE()->current_frame ; frame != 0 ; frame = frame->prev) {
	if (frame->current_method != 0) {
	    n++;
	} 
    }

    /* allocate the array */
    ctx = (HArrayOfObject *)ArrayAlloc(T_CLASS, n);
    if (ctx == 0) {
	SignalError(0, JAVAPKG "OutOfMemoryError", 0);
	return 0;
    }
    ctxdata = (HObject **)unhand(ctx)->body;
    ctxdata[n] = (HObject *)classJavaLangClass;

    /* fill the array */
    for (n = 0, frame = EE()->current_frame ; frame != 0 ; frame = frame->prev) {
	if (frame->current_method != 0) {
	    ctxdata[n++] = (HObject *)(fieldclass(&frame->current_method->fb)->HandleToSelf);
	}
    }

    return ctx;
}

Hjava_lang_ClassLoader *
java_lang_SecurityManager_currentClassLoader(Hjava_lang_SecurityManager *this)
{
    struct javaframe *frame;
    ClassClass	*cb;

    for (frame = EE()->current_frame ; frame != 0 ; frame = frame->prev) {
	if (frame->current_method != 0) {
	    cb = fieldclass(&frame->current_method->fb);
	    if (cb && cbLoader(cb)) {
		return (Hjava_lang_ClassLoader *)cbLoader(cb);
	    }
	} 
    }

    return 0;
}

long
java_lang_SecurityManager_classDepth(Hjava_lang_SecurityManager *this, HString *name)
{
    struct javaframe *frame;
    ClassClass	*cb;
    char buf[128], *p;
    int depth = 0;

    javaString2CString(name, buf, sizeof(buf));
    for (p = buf ; *p ; p++) {
	if (*p == '.') {
	    *p = '/';
	}
    }

    for (frame = EE()->current_frame ; frame != 0 ; frame = frame->prev, depth++) {
	if (frame->current_method != 0) {
	    cb = fieldclass(&frame->current_method->fb);
	    if (cb && !strcmp(classname(cb), buf)) {
		return depth;
	    }
	} 
    }
    return -1;
}

long
java_lang_SecurityManager_classLoaderDepth(Hjava_lang_SecurityManager *this)
{
    struct javaframe *frame;
    ClassClass	*cb;
    int depth = 0;

    for (frame = EE()->current_frame ; frame != 0 ; frame = frame->prev, depth++) {
	if (frame->current_method != 0) {
	    cb = fieldclass(&frame->current_method->fb);
	    if (cb && cbLoader(cb)) {
		return depth;
	    }
	} 
    }
    return -1;
}
