/*
 * @(#)appletviewer.c	1.9 95/11/28
 *
 * Copyright (c) 1995 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

#include <windows.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>

#ifdef DEBUG
#define JAVA "java_g"
#else
#define JAVA "java"
#endif

#ifndef S_ISDIR
#define S_ISDIR(mode) (((mode) & _S_IFDIR) == _S_IFDIR)
#endif

#ifdef WINMAIN
#define main _main
__declspec(dllimport) char **__initenv;
#endif

void *
emalloc(size_t size)
{
    void *p = malloc(size);
    if (p == 0) {
	fprintf(stderr, "*** Not enough memory to start applet viewer ***\n");
	exit(1);
    }
}

void
set_class_path(void)
{
    char *cp = getenv("CLASSPATH");
    char loaddir[MAX_PATH];
    char *s;

    GetModuleFileName(0, loaddir, MAX_PATH);
    *(strrchr(loaddir, '\\')) = 0;

    if (cp == 0) {
	s = emalloc(strlen(loaddir) * 2 + 64);
	sprintf(s, "CLASSPATH=%s\\..\\classes;%s\\..\\lib\\classes.zip",
		loaddir, loaddir);
    } else {
	s = emalloc(strlen(cp) + strlen(loaddir) * 2 + 64);
	sprintf(s, "CLASSPATH=%s;%s\\..\\classes;%s\\..\\lib\\classes.zip",
		cp, loaddir, loaddir);
    }
    putenv(s);
}

int
main(int argc, char *argv[])
{
    char **newargv, **ap;
    char *java_home, *user_home, *hotjava_dir;
    struct stat sbuf;
    int len;

    if (argc < 2) {
        fprintf(stderr, "usage: appletviewer [-debug] html_file\n");
	exit(1);
    }

    set_class_path();

    if ((java_home = getenv("JAVA_HOME")) == 0) {
        char loaddir[MAX_PATH];
        GetModuleFileName(0, loaddir, MAX_PATH);
	*(strrchr(loaddir, '\\')) = 0;
	java_home = emalloc(strlen(loaddir) + sizeof("\\..") + 1);
	sprintf(java_home, "%s\\..", loaddir);
    } else {
	len = strlen(java_home);
	if (len == 0 || stat(java_home, &sbuf) != 0 || !S_ISDIR(sbuf.st_mode)) {
	    fprintf(stderr, "Invalid JAVA_HOME: %s\n", java_home);
	    exit(1);
	}
    }

    if ((user_home = getenv("HOME")) == 0) {
	char *drive = getenv("HOMEDRIVE");
	char *path  = getenv("HOMEPATH");
	if (drive != 0 && path != 0) {
	    user_home = emalloc(strlen(drive) + strlen(path) + 1);
	    sprintf(user_home, "%s%s", drive, path);
	} else {
	    user_home = java_home;
	}
    }

    len = strlen(user_home);
    if (len == 0 || stat(user_home, &sbuf) != 0 || !S_ISDIR(sbuf.st_mode)) {
	fprintf(stderr, "Invalid home directory: %s\n", user_home);
	exit(1);
    }

    hotjava_dir = emalloc(strlen(user_home) + sizeof("\\.hotjava") + 1);
    if (user_home[len - 1] == '\\' || user_home[len - 1] == ':') {
	sprintf(hotjava_dir, "%s.hotjava", user_home);
    } else {
	sprintf(hotjava_dir, "%s\\.hotjava", user_home);
    }

    if (stat(hotjava_dir, &sbuf) != 0) {
	fprintf(stderr, "Creating %s directory...\n", hotjava_dir);
	if (mkdir(hotjava_dir) == -1) {
	    perror(hotjava_dir);
	    exit(1);
	}
    }

    newargv = ap = emalloc((argc + 4) * sizeof(char *));
    *ap++ = JAVA;
    *ap = emalloc(strlen(java_home) + 16);
    sprintf(*ap++, "-Djava.home=%s", java_home);
    if (strcmp(*++argv, "-debug") == 0) {
        *ap++ = "sun.tools.ttydebug.TTY";
	argv++;
    }
    *ap++ = "sun.applet.AppletViewer";
    do {
	*ap++ = *argv++;
    } while (*argv != 0);

    exit(java_main(ap - newargv, newargv));
}

#ifdef WINMAIN

int WINAPI
WinMain(HINSTANCE inst, HINSTANCE previnst, LPSTR cmdline, int cmdshow)
{
    __initenv = _environ;
    main(__argc, __argv);
}

#endif
