/*
 * @(#)gc_md.c	1.12 95/11/29
 *
 * Copyright (c) 1995 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/*
 * Win32 support for garbage collecter
 */

#include <windows.h>
#include <stdio.h>

#include "threads.h"
#include "threads_md.h"

/*
 * Get context of suspended thread and copy current register contents
 * into reserved area at base of thread's stack so that they can be
 * subsequently traced by the garbage collector.
 */
static int
saveContextInfo(sys_thread_t *tid, void *self)
{
    /* REMIND: Fix for Win95 */
    if (tid != (sys_thread_t *)self && tid->stack_base != 0) {
	CONTEXT context;
	DWORD *esp = (DWORD *)tid->stack_base - N_TRACED_REGS;

	sysAssert(tid->state == SUSPENDED);
	context.ContextFlags = CONTEXT_INTEGER;
	if (!GetThreadContext(tid->handle, &context)) {
	    fprintf(stderr, "GetThreadContext failed (errcode = %x)\n",
		    GetLastError());
	    return SYS_ERR;
	}
	*esp++ = context.Eax;
	*esp++ = context.Ebx;
	*esp++ = context.Ecx;
	*esp++ = context.Edx;
	*esp++ = context.Esi;
	*esp   = context.Edi;
    }
    return SYS_OK;
}

void
gc_locked(int acall, unsigned int free)
{
    sysThreadEnumerateOver(saveContextInfo, (void *)sysThreadSelf());
    gc0_locked(acall, free);
}
