/*
 * @(#)io_md.c	1.13 95/11/16
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

#include <io.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "path_md.h"
#include "io_md.h"
#include "sys_api.h"
#include "dirent.h"

#define IO_EXCEPTION "java/io/IOException"

int
sysOpen(const char *path, int oflag, int mode)
{
    char pathbuf[MAX_PATH];

    if (strlen(path) > MAX_PATH - 1) {
	errno = ENAMETOOLONG;
	return -1;
    }
    return open(sysNativePath(strcpy(pathbuf, path)), oflag | O_BINARY, mode);
}

int
sysAccess(const char *path, int mode)
{
    char pathbuf[MAX_PATH];

    if (strlen(path) > MAX_PATH - 1) {
	errno = ENAMETOOLONG;
	return -1;
    }
    return access(sysNativePath(strcpy(pathbuf, path)), mode);
}

int
sysStat(const char *path, struct stat *sbuf)
{
    char pathbuf[MAX_PATH];

    if (strlen(path) > MAX_PATH - 1) {
	errno = ENAMETOOLONG;
	return -1;
    }
    return stat(sysNativePath(strcpy(pathbuf, path)), sbuf);
}

int
sysUnlink(const char *path)
{
    char pathbuf[MAX_PATH];

    if (strlen(path) > MAX_PATH - 1) {
	errno = ENAMETOOLONG;
	return -1;
    }
    return unlink(sysNativePath(strcpy(pathbuf, path)));
}

int
sysMkdir(const char *path, int mode)
{
    char pathbuf[MAX_PATH];

    if (strlen(path) > MAX_PATH - 1) {
	errno = ENAMETOOLONG;
	return -1;
    }
    return mkdir(sysNativePath(strcpy(pathbuf, path)));
}

DIR *
sysOpenDir(const char *path)
{
    char pathbuf[MAX_PATH];

    if (strlen(path) > MAX_PATH - 1) {
       errno = ENAMETOOLONG;
       return 0;
    }
    return opendir(sysNativePath(strcpy(pathbuf, path)));
}

int
sysAvailable(int fd, long *pbytes)
{
    long cur, end;
    struct stat stbuf;

    if (fstat(fd, &stbuf) >= 0) {
        int mode = stbuf.st_mode;
        if (S_ISCHR(mode) || S_ISFIFO(mode)) {
	    return FALSE; /* Currently unsupported */
        }
    }
    if ((cur = lseek(fd, 0L, SEEK_CUR)) == -1) {
	return FALSE;
    } else if ((end = lseek(fd, 0L, SEEK_END)) == -1) {
	return FALSE;
    } else if (lseek(fd, cur, SEEK_SET) == -1) {
	return FALSE;
    }
    *pbytes = end - cur;
    return TRUE;
}
