/*
 * @(#)math_md.c	1.12 95/10/05
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

#include <windows.h>
#include <stdlib.h>
#include <ctype.h>
#include <math.h>

double
rint(double a)
{
    /*
     * For now, this is adequate but we should really be calling the
     * appropriate inline FP87 instruction in order to get the correct
     * mode.
     */
    return floor(a + 0.5);
}

/*
 * Implement the remainder operation for floating point.  Since fmod()
 * doesn't do the correct thing for infinity in the divisor, we have
 * to patch up the return values in some of those cases.
 */
double
drem(double dividend, double divisor) {
    if (!_finite(divisor)) {
	if (_isnan(divisor))
	    return divisor;
	if (_finite(dividend))
	    return dividend;
    }
    return fmod(dividend, divisor);
}

/*
 * Implement the IEEE remainder operation as per IEEE 754.  Since fmod()
 * doesn't do the correct thing for infinity in the divisor, we have
 * to patch up the return values in some of those cases.  Then we have
 * to adjust the domain of the results to be -|d/2| <= rem <= |d/2|.
 */
double
ieeerem(double dividend, double divisor) {
    if (!_finite(divisor)) {
	if (_isnan(divisor))
	    return divisor;
	if (_finite(dividend))
	    return dividend;
    }
    {
	double q = fmod(dividend, divisor);
	double d = fabs(divisor);
	if (q < 0) {
	    if (-q > d / 2)
		q += d;
	} else {
	    if (q > d / 2)
		q -= d;
	}
	return q;
    }
}
