/*
 * @(#)path_md.c	1.14 95/11/20 David Connelly
 *
 * Copyright (c) 1995 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/*
 * Machine dependent path name and file name manipulation code
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <sys/stat.h>

#include "path.h"	/* for cpe_t */
#include "zip.h"	/* for zip_t */
#include "io_md.h"	/* for S_ISREG */
#include "bool.h"

cpe_t **
sysGetClassPath(void)
{
    static cpe_t **classpath;

    if (classpath == 0) {
	char *cps, *s;
	int ncpe = 1;
	cpe_t **cpp;
	if ((cps = getenv("CLASSPATH")) == 0) {
	    cps = ".;classes";
	}
	cps = strdup(cps);
	for (s = cps; *s != '\0'; s++) {
	    if (*s == ';') {
		ncpe++;
	    }
	}
	cpp = classpath = malloc((ncpe + 1) * sizeof(cpe_t *));
	while (cps != 0) {
	    struct stat sbuf;
	    char *path = cps;
	    if ((cps = strchr(cps, ';')) != 0) {
		*cps++ = '\0';
	    }
	    if (*path == '\0') {
		path = ".";
	    }
	    if (stat(path, &sbuf) == 0 && S_ISREG(sbuf.st_mode)) {
		/* Better be a zip file */
		zip_t *zip = zip_open(path);
		if (zip != 0) {
		    cpe_t *cpe = malloc(sizeof(cpe_t));
		    cpe->type = CPE_ZIP;
		    cpe->u.zip = zip;
		    *cpp++ = cpe;
		}
	    } else {
		/* Better be a directory */
		cpe_t *cpe = malloc(sizeof(cpe_t));
		cpe->type = CPE_DIR;
		cpe->u.dir = path;
		*cpp++ = cpe;
	    }
	}
	*cpp = 0;
    }
    return classpath;
}

#define isfilesep(c) ((c) == '/' || (c) == '\\')

char *
sysNativePath(char *path)
{
    char *src = path, *dst = path;
    int len = 0;

    /*
     * Remove leading separator if followed by drive designator. This hack
     * is necessary to support file URLs containing drive letter specifiers
     * (i.e. "file://c:/path"). As an additional side effect, "/c:/path" can
     * now be used as an alternative way of specifying "c:/path".
     */
    if (isfilesep(*src)) {
	while (isfilesep(*src)) {
	    src++;
	}
	if (!(isalpha(*src) && src[1] == ':')) {
	    *dst++ = '\\'; len++;
	}
    }

    /*
     * Remove redundant separators from path.
     */
    while (*src != '\0') {
	if (isfilesep(*src)) {
	    if (len == 0 || dst[-1] != '\\') {
		*dst++ = '\\'; len++;
	    }
	    src++;
	} else {
	    *dst++ = *src++; len++;
	}
    }

    /*
     * If not root directory then remove trailing separator.
     */
    if (len > 1 && dst[-1] == '\\') {
	if (!(len == 3 && isalpha(dst[-3]) && dst[-2] == ':')) {
	    --dst; --len;
	}
    }

    *dst = '\0';
    return path;
}
