/*
 * @(#)process_md.c	1.9 95/11/30 David Connelly
 *
 * Copyright (c) 1995 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

#include <windows.h>
#include <stdio.h>
#include <errno.h>
#include <ctype.h>

#include "java_io_FileDescriptor.h"
#include "java_io_FileInputStream.h"
#include "java_lang_System.h"
#include "java_lang_Runtime.h"
#include "java_lang_Process.h"
#include "java_lang_Win32Process.h"

#define PIPE_SIZE 512

void
java_lang_Win32Process_create(Hjava_lang_Win32Process *Hthis, HString *Hcmd,
			      HString *Henv)
{
    Classjava_lang_Win32Process *this = unhand(Hthis);
    Classjava_io_FileDescriptor *in  = unhand(this->stdin_fd);
    Classjava_io_FileDescriptor *out = unhand(this->stdout_fd);
    Classjava_io_FileDescriptor *err = unhand(this->stderr_fd);
    HANDLE hread[3], hwrite[3];
    SECURITY_ATTRIBUTES sa;
    PROCESS_INFORMATION pi;
    STARTUPINFO si;
    char *cmd, *env;
    int ret;

    if (Hcmd == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    sa.nLength = sizeof(sa);
    sa.lpSecurityDescriptor = 0;
    sa.bInheritHandle = TRUE;

    if (!(CreatePipe(&hread[0], &hwrite[0], &sa, PIPE_SIZE) &&
          CreatePipe(&hread[1], &hwrite[1], &sa, PIPE_SIZE) &&
	  CreatePipe(&hread[2], &hwrite[2], &sa, PIPE_SIZE))) {
	SignalError(0, "java/io/IOException", "CreatePipe");
	return;
    }

    cmd = allocCString(Hcmd);
    env = (Henv != 0) ? allocCString(Henv) : 0;

    memset(&si, 0, sizeof(si));
    si.cb = sizeof(si);
    si.dwFlags = STARTF_USESTDHANDLES;
    si.hStdInput = hread[0];
    si.hStdOutput = hwrite[1];
    si.hStdError = hwrite[2];

    ret = CreateProcess(0, cmd, 0, 0, TRUE, 0, env, 0, &si, &pi);
    if (!ret) {
	char *s;
	fprintf(stderr, "CreateProcess: ");
	for (s = cmd; *s && !isspace(*s); s++) {
	    putc(*s, stderr);
	}
	fprintf(stderr, ": Could not execute (err = %x)\n", GetLastError());
    }
    CloseHandle(hread[0]);
    CloseHandle(hwrite[1]);
    CloseHandle(hwrite[2]);
    free(cmd);

    if (!ret) {
	CloseHandle(hwrite[0]);
	CloseHandle(hread[1]);
	CloseHandle(hread[2]);
	SignalError(0, "java/io/IOException", "CreateProcess");
	return;
    }

    CloseHandle(pi.hThread);
    this->handle = (long)pi.hProcess;
    in->fd = _open_osfhandle(hwrite[0])+1;
    out->fd = _open_osfhandle(hread[1])+1;
    err->fd = _open_osfhandle(hread[2])+1;
}

long
java_lang_Win32Process_exitValue(Hjava_lang_Win32Process *Hthis)
{
    long exit_code;

    GetExitCodeProcess((HANDLE)unhand(Hthis)->handle, &exit_code);
    return exit_code;
}

long
java_lang_Win32Process_waitFor(Hjava_lang_Win32Process *Hthis)
{
    Classjava_lang_Win32Process *this = unhand(Hthis);
    long exit_code;

    WaitForSingleObject((HANDLE)this->handle, INFINITE);
    GetExitCodeProcess((HANDLE)this->handle, &exit_code);
    return exit_code; 
}

void
java_lang_Win32Process_destroy(Hjava_lang_Win32Process *Hthis)
{
    TerminateProcess((HANDLE)unhand(Hthis)->handle, 1);
}

void
java_lang_Win32Process_close(Hjava_lang_Win32Process *Hthis)
{
    CloseHandle((HANDLE)unhand(Hthis)->handle);
}

Hjava_lang_Process *
java_lang_Runtime_execInternal(Hjava_lang_Runtime *Hthis, 
			       HArrayOfString *Hcmd, HArrayOfString *Henv)
{
    Hjava_lang_Process *Hproc;
    ClassClass *cb;

    if (Hcmd == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return 0;
    }
    Hproc = (Hjava_lang_Process *)
	    execute_java_constructor(EE(), JAVAPKG "Win32Process", 0,
				     "([Ljava/lang/String;[Ljava/lang/String;)",
				     Hcmd, Henv);
    return Hproc;
}
