/*
 * @(#)properties_md.c	1.13 95/12/03
 *
 * Copyright (c) 1995 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "sys_api.h"
#include "oobj.h"
#include "path.h"

#include "java_lang_System.h"

#ifndef VER_PLATFORM_WIN32_WINDOWS
#define VER_PLATFORM_WIN32_WINDOWS 1
#endif

char **user_props = NULL;

/*
 * Add a new property.
 */
static void
putprop(struct Hjava_util_Properties *props, char *key, char *val)
{
    execute_java_dynamic_method(0, (HObject *)props, "put",
		    "(Ljava/lang/Object;Ljava/lang/Object;)Ljava/lang/Object;",
		    makeJavaString(key, strlen(key)),
		    makeJavaString(val, strlen(val)));
}

/*
 * Initialize basic properties.
 */
struct Hjava_util_Properties *
java_lang_System_initProperties(struct Hjava_lang_System *this, 
				struct Hjava_util_Properties *props) 
{
    char *java_home, *v;
    char buf[max(512, MAX_PATH+1)];
    OSVERSIONINFO ver;

    /* Java home directory */
    if ((v = getenv("JAVA_HOME")) != 0) {
	if (jio_snprintf(buf, sizeof(buf), "%s", v) != -1) {
	    v = sysNativePath(buf);
	} else {
	    v = 0;
	}
    }
    if (v == 0) {
	GetModuleFileName(NULL, buf, MAX_PATH);
	strcpy(strrchr(buf, '\\'), "\\..");
	v = buf;
    }
    java_home = strdup(v);
    putprop(props, "java.home", java_home);

    /* Java properties */
    putprop(props, "java.version", RELEASE);
    putprop(props, "java.vendor", "Sun Microsystems Inc.");
    putprop(props, "java.vendor.url", "http://www.sun.com/");
    jio_snprintf(buf, sizeof(buf), "%d.%d", JAVA_VERSION, JAVA_MINOR_VERSION);
    putprop(props, "java.class.version", buf);
    v = getenv("CLASSPATH");
    putprop(props, "java.class.path", v != 0 ? v : "");

    /* AWT properties */
    putprop(props, "awt.toolkit", "sun.awt.win32.MToolkit");

    /* OS properties */
    ver.dwOSVersionInfoSize = sizeof(ver);
    GetVersionEx(&ver);
    switch (ver.dwPlatformId) {
	case VER_PLATFORM_WIN32s:
	    putprop(props, "os.name", "Windows 3.1");
	    break;
	case VER_PLATFORM_WIN32_WINDOWS:
	    putprop(props, "os.name", "Windows 95");
	    break;
	case VER_PLATFORM_WIN32_NT:
	    putprop(props, "os.name", "Windows NT");
	    break;
    }
    jio_snprintf(buf, sizeof(buf), "%d.%d",
		 ver.dwMajorVersion, ver.dwMinorVersion);
    putprop(props, "os.version", buf);
#if _X86_
    putprop(props, "os.arch", "x86");
#else
    putprop(props, "os.arch", "unknown");
#endif

    /* File system properties */
    putprop(props, "file.separator", "\\");
    putprop(props, "path.separator", ";");
    putprop(props, "line.separator", "\r\n");

    /* User name */
    if ((v = getenv("USER")) == 0) {
	int buflen = sizeof(buf);
	v = GetUserName(buf, &buflen) ? buf : "unknown";
    }
    putprop(props, "user.name", v);

    /* Home directory */
    if ((v = getenv("HOME")) != 0) {
	if (jio_snprintf(buf, sizeof(buf), "%s", v) != -1) {
	    v = sysNativePath(buf);
	} else {
	    v = 0;
	}
    }
    if (v == 0) {
	char *drive = getenv("HOMEDRIVE");
	char *path  = getenv("HOMEPATH");
	if (drive != 0 && path != 0) {
	    if (jio_snprintf(buf, sizeof(buf), "%s%s", drive, path) != -1) {
		v = sysNativePath(buf);
	    } else {
		v = 0;
	    }
	}
    }
    putprop(props, "user.home", v == 0 ? java_home : v);

    /* Current directory */
    GetCurrentDirectory(sizeof(buf), buf);
    putprop(props, "user.dir", buf);

    /* User defined properties, set in javai.c */
    if (user_props != NULL) {
        int i;
        for (i = 0; user_props[i] && user_props[i+1]; i += 2) {
            putprop(props, user_props[i], user_props[i+1]);
        }
    }

    free(java_home);
    return props;
}
