/*
 * @(#)system_md.c	1.16 95/11/29
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

#include <windows.h>
#include <time.h>
#include <sys/types.h>
#include <sys/timeb.h>

#include "sys_api.h"

long
sysGetMilliTicks(void)
{
    return(GetTickCount());
}


long
sysTime(long *l_p)
{
    return(time(l_p));
}

#define FT2INT64(ft) \
	((int64_t)(ft).dwHighDateTime << 32 | (int64_t)(ft).dwLowDateTime)

int64_t
sysTimeMillis(void)
{
    SYSTEMTIME st1, st0;
    FILETIME   ft1, ft0;

    memset(&st0, 0, sizeof(st0));
    st0.wYear  = 1970;
    st0.wMonth = 1;
    st0.wDay   = 1;
    SystemTimeToFileTime(&st0, &ft0);

    GetSystemTime(&st1);
    SystemTimeToFileTime(&st1, &ft1);

    return (FT2INT64(ft1) - FT2INT64(ft0)) / 10000;
}

void
sysStrftime(char *buf, int buflen, char *fmt, struct tm *t)
{
    strftime(buf, buflen, fmt, t);
}

time_t sysMktime(struct tm *t)
{
    return mktime(t);
}

/*
 * Moved here as a portability concern - not availiable on all platforms
 *
 * The API to this is historical, and isn't exactly what we want, but
 * we can check for buffer overflow regardless.  The caller should be
 * able to guarantee against overflow, but if so, truncate silently.
 */
int jio_snprintf(char *str, size_t count, const char *fmt, ...);

void ll2str(int64_t number, char *s, char *limit)
{
#define MAGIC    10000000	/* slightly more than (2^64) ^ (1/3) */
    int group1, group2, group3;
    char *sign;
    int len = limit - s;

    if (number >= 0) {
	sign = "";
        number = -number;	/* this lets -2^63 still work! */
    } else {
	sign = "-";
    }

    group3 = -(number % MAGIC);	/* low 7 digits */
    number = (number + group3)/MAGIC;
    group2 = -(number % MAGIC);	/* next 7 digits */
    number = (number + group2)/MAGIC;
    group1 = -number;		/* high 7 digits */
    if (group1 != 0) {
	(void) jio_snprintf(s, len, "%s%d%07.7d%07.7d",
			    sign, group1, group2, group3);
    } else if (group2 != 0) {
	(void) jio_snprintf(s, len, "%s%d%07.7d", sign, group2, group3);
    } else {
	(void) jio_snprintf(s, len, "%s%d", sign, group3);
    }
}

/*
 * THE FOLLOWING DEFINITION IS A FUNCTION CALL IN ORDER TO WORK AROUND
 * AN OPTIMIZER BUG IN MSVC++ 2.1 (see typedefs_md.h).
 */
int64_t ll_shl(int64_t i, int bits)
{
    return i << bits;
}

void *
sysAllocateMem(long size)
{
    return malloc(size);
}

typedef void (*pProc)(void);

static struct exit_proc {
    pProc proc;
    struct exit_proc *next;
} *exit_procs = NULL;

void
sysExit(int status)
{
    /* The only difference between this and Win32's exit is that 
     * this version is invoked before any threads get killed.
     */
    struct exit_proc *pExit = exit_procs;
    while (pExit != NULL) {
        struct exit_proc *pNext = pExit->next;
        (*pExit->proc)();
        sysFree(pExit);
        pExit = pNext;
    }
    exit(status);
}

int
sysAtexit(void (*func)(void))
{
    struct exit_proc *pExit = sysMalloc(sizeof(struct exit_proc));
    if (pExit == NULL) {
        return 1;
    }
    pExit->proc = func;
    pExit->next = exit_procs;
    exit_procs = pExit;
    return 0;
}

void
sysAbort(void)
{
    abort();
}

/*
 * On Win32 systems, gmtime and localtime use thread-specific data to store
 * the result so both are already MT-safe. 
 */

struct tm *
sysGmtime(time_t *clock, struct tm *res)
{
    struct tm *tm = gmtime(clock);

    if (tm != NULL) *res = *tm;
    return res;
}

struct tm *
sysLocaltime(time_t *clock, struct tm *res)
{
    struct tm *tm = localtime(clock);

    if (tm != NULL) *res = *tm;
    return res;
}
