/*
 * @(#)agentthread.c	1.6 95/11/20 Thomas Ball
 *
 * Copyright (c) 1995 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/*-
 *	Win32-specific routines to support the debugging agent.  
 */

#include <oobj.h>
#include <threads.h>
#include <threads_md.h>
#include <opcodes.h>

#include "sun_tools_debug_Agent.h"
#include "sun_tools_debug_AgentConstants.h"

void setDebugState(void) {
    static int fInitialized = 0;

    if (!fInitialized) {
	if (getenv("ENTER_DBX") != NULL) {
            _asm int 3;
        }
	fInitialized = 1;
    }
}

long
sun_tools_debug_Agent_getThreadStatus(Hsun_tools_debug_Agent *this, 
				       Hjava_lang_Thread *h)
{
    if (h == 0)
	return sun_tools_debug_AgentConstants_THR_STATUS_UNKNOWN;

    if (SYSTHREAD(h) == 0)
	return sun_tools_debug_AgentConstants_THR_STATUS_ZOMBIE;

    switch (SYSTHREAD(h)->state) {
      case RUNNABLE:
	return sun_tools_debug_AgentConstants_THR_STATUS_RUNNING;

      case SUSPENDED: {
	ExecEnv* thread_ee = (ExecEnv *)unhand(h)->eetop;
	if ((thread_ee != 0) && (thread_ee->current_frame != 0) &&
            (thread_ee->current_frame->lastpc != 0) &&
	    (*thread_ee->current_frame->lastpc == opc_breakpoint)) {
	    return sun_tools_debug_AgentConstants_THR_STATUS_BREAK;
	}
	return sun_tools_debug_AgentConstants_THR_STATUS_SUSPENDED;
      }

      case SLEEPING:
	return sun_tools_debug_AgentConstants_THR_STATUS_SLEEPING;

      case MONITOR_WAIT:
	return sun_tools_debug_AgentConstants_THR_STATUS_MONWAIT;
    
      case CONDVAR_WAIT:
	return sun_tools_debug_AgentConstants_THR_STATUS_CONDWAIT;

      default:
        return sun_tools_debug_AgentConstants_THR_STATUS_RUNNING;
    }
}
